@@ -138,8 +138,10 @@ QRect HexagonalRenderer::boundingRect(const QRect &rect) const
 }
 
 void HexagonalRenderer::drawGrid(QPainter *painter, const QRectF &exposed,
-                                 QColor gridColor) const
+                                 QColor gridColor, QSize gridMajor) const
 {
+    Q_UNUSED(gridMajor)  // Unclear how this should apply to hexagonal maps
+
     QRect rect = exposed.toAlignedRect();
     if (rect.isNull())
         return;
@@ -184,7 +186,9 @@ void HexagonalRenderer::drawGrid(QPainter *painter, const QRectF &exposed,
     QVector<QLine> lines;
     lines.reserve(8);
 
-    QPen gridPen = makeGridPen(painter->device(), gridColor);
+    QPen _gridPen, gridPen; // always use major grid pen for hex maps
+    setupGridPens(painter->device(), gridColor, _gridPen, gridPen, p.tileWidth, gridMajor);
+
     painter->setPen(gridPen);
 
     if (p.staggerX) {
@@ -266,84 +270,65 @@ void HexagonalRenderer::drawGrid(QPainter *painter, const QRectF &exposed,
     }
 }
 
-void HexagonalRenderer::drawTileLayer(QPainter *painter,
-                                      const TileLayer *layer,
-                                      const QRectF &exposed) const
+QPointF HexagonalRenderer::snapToGrid(const QPointF &pixelCoords, int subdivisions) const
 {
-    CellRenderer renderer(painter, this, layer->effectiveTintColor(), CellRenderer::HexagonalCells);
-    auto tileRenderFunction = [&renderer](const Cell &cell, const QPointF &pos, const QSizeF &size) {
-        renderer.render(cell, pos, size, CellRenderer::BottomLeft);
-    };
-    drawTileLayer(layer, tileRenderFunction, exposed);
+    const QPoint tileCoords = pixelToTileCoords(pixelCoords).toPoint();
+    QPolygonF hex = tileToScreenPolygon(tileCoords);
+
+    if (subdivisions > 1) {
+        const QPointF center = (hex[0] + hex[4]) / 2;
+        hex.append(center);
+    }
+
+    QPointF nearest;
+    qreal minDist = std::numeric_limits<qreal>::max();
+
+    for (const QPointF &candidate : hex) {
+        const QPointF diff = candidate - pixelCoords;
+        const qreal lengthSquared = diff.x() * diff.x() + diff.y() * diff.y();
+        if (lengthSquared < minDist) {
+            minDist = lengthSquared;
+            nearest = candidate;
+        }
+    }
+
+    return nearest;
 }
 
-void HexagonalRenderer::drawTileLayer(const TileLayer *layer,
-                                      const RenderTileCallback &renderTile,
+void HexagonalRenderer::drawTileLayer(const RenderTileCallback &renderTile,
                                       const QRectF &exposed) const
 {
     const RenderParams p(map());
 
-    QRect rect = exposed.toAlignedRect();
-
-    if (rect.isNull())
-        rect = boundingRect(layer->bounds());
-
-    QMargins drawMargins = layer->drawMargins();
-    drawMargins.setBottom(drawMargins.bottom() + p.tileHeight);
-    drawMargins.setRight(drawMargins.right() - p.tileWidth);
-
-    rect.adjust(-drawMargins.right(),
-                -drawMargins.bottom(),
-                drawMargins.left(),
-                drawMargins.top());
-
     // Determine the tile and pixel coordinates to start at
-    QPoint startTile = screenToTileCoords(rect.topLeft()).toPoint();
-
-    // Compensate for the layer position
-    startTile -= layer->position();
-
-    QPoint startPos = tileToScreenCoords(startTile + layer->position()).toPoint();
+    QPoint startTile = screenToTileCoords(exposed.topLeft()).toPoint();
+    QPoint startPos = tileToScreenCoords(startTile).toPoint();
 
     /* Determine in which half of the tile the top-left corner of the area we
      * need to draw is. If we're in the upper half, we need to start one row
      * up due to those tiles being visible as well. How we go up one row
      * depends on whether we're in the left or right half of the tile.
      */
-    const bool inUpperHalf = rect.y() - startPos.y() < p.sideOffsetY;
-    const bool inLeftHalf = rect.x() - startPos.x() < p.sideOffsetX;
+    const bool inUpperHalf = exposed.y() - startPos.y() < p.sideOffsetY;
+    const bool inLeftHalf = exposed.x() - startPos.x() < p.sideOffsetX;
 
     if (inUpperHalf)
         startTile.ry()--;
     if (inLeftHalf)
         startTile.rx()--;
 
-    const int endX = map()->infinite() ? layer->bounds().right() - layer->x() + 1 : layer->width();
-    const int endY = map()->infinite() ? layer->bounds().bottom() - layer->y() + 1 : layer->height();
-
     if (p.staggerX) {
-        if (!map()->infinite()) {
-            startTile.setX(qMax(-1, startTile.x()));
-            startTile.setY(qMax(-1, startTile.y()));
-        }
-
-        startPos = tileToScreenCoords(startTile + layer->position()).toPoint();
+        startPos = tileToScreenCoords(startTile).toPoint();
         startPos.ry() += p.tileHeight;
 
-        bool staggeredRow = p.doStaggerX(startTile.x() + layer->x());
+        bool staggeredRow = p.doStaggerX(startTile.x());
 
-        for (; startPos.y() < rect.bottom() && startTile.y() < endY;) {
+        while (startPos.y() - p.tileHeight < exposed.bottom()) {
             QPoint rowTile = startTile;
             QPoint rowPos = startPos;
 
-            for (; rowPos.x() < rect.right() && rowTile.x() < endX; rowTile.rx() += 2) {
-                const Cell &cell = layer->cellAt(rowTile);
-
-                if (!cell.isEmpty()) {
-                    const Tile *tile = cell.tile();
-                    const QSize size = tile ? tile->size() : map()->tileSize();
-                    renderTile(cell, rowPos, size);
-                }
+            for (; rowPos.x() < exposed.right(); rowTile.rx() += 2) {
+                renderTile(rowTile, rowPos);
 
                 rowPos.rx() += p.tileWidth + p.sideLengthX;
             }
@@ -362,33 +347,22 @@ void HexagonalRenderer::drawTileLayer(const TileLayer *layer,
             startPos.ry() += p.rowHeight;
         }
     } else {
-        if (!map()->infinite()) {
-            startTile.setX(qMax(0, startTile.x()));
-            startTile.setY(qMax(0, startTile.y()));
-        }
-
-        startPos = tileToScreenCoords(startTile + layer->position()).toPoint();
+        startPos = tileToScreenCoords(startTile).toPoint();
         startPos.ry() += p.tileHeight;
 
         // Odd row shifting is applied in the rendering loop, so un-apply it here
-        if (p.doStaggerY(startTile.y() + layer->y()))
+        if (p.doStaggerY(startTile.y()))
             startPos.rx() -= p.columnWidth;
 
-        for (; startPos.y() < rect.bottom() && startTile.y() < endY; startTile.ry()++) {
+        for (; startPos.y() - p.tileHeight < exposed.bottom(); startTile.ry()++) {
             QPoint rowTile = startTile;
             QPoint rowPos = startPos;
 
-            if (p.doStaggerY(startTile.y() + layer->y()))
+            if (p.doStaggerY(startTile.y()))
                 rowPos.rx() += p.columnWidth;
 
-            for (; rowPos.x() < rect.right() && rowTile.x() < endX; rowTile.rx()++) {
-                const Cell &cell = layer->cellAt(rowTile);
-
-                if (!cell.isEmpty()) {
-                    const Tile *tile = cell.tile();
-                    const QSize size = tile ? tile->size() : map()->tileSize();
-                    renderTile(cell, rowPos, size);
-                }
+            for (; rowPos.x() < exposed.right(); rowTile.rx()++) {
+                renderTile(rowTile, rowPos);
 
                 rowPos.rx() += p.tileWidth + p.sideLengthX;
             }
@@ -493,24 +467,24 @@ QPointF HexagonalRenderer::screenToTileCoords(qreal x, qreal y) const
     }
 
     int nearest = 0;
-    qreal minDist = std::numeric_limits<qreal>::max();
+    float minDist = std::numeric_limits<float>::max();
 
     for (int i = 0; i < 4; ++i) {
         const QVector2D &center = centers[i];
-        const qreal dc = (center - rel).lengthSquared();
+        const float dc = (center - rel).lengthSquared();
         if (dc < minDist) {
             minDist = dc;
             nearest = i;
         }
     }
 
-    static const QPoint offsetsStaggerX[4] = {
+    static constexpr QPoint offsetsStaggerX[4] = {
         QPoint( 0,  0),
         QPoint(+1, -1),
         QPoint(+1,  0),
         QPoint(+2,  0),
     };
-    static const QPoint offsetsStaggerY[4] = {
+    static constexpr QPoint offsetsStaggerY[4] = {
         QPoint( 0,  0),
         QPoint(-1, +1),
         QPoint( 0, +1),
