@@ -38,6 +38,8 @@
 #include <QString>
 #include <QVector>
 
+#include <iterator>
+
 namespace Tiled {
 
 class GroupLayer;
@@ -184,9 +186,12 @@ public:
 
     void setOffset(const QPointF &offset);
     QPointF offset() const;
-
     QPointF totalOffset() const;
 
+    void setParallaxFactor(const QPointF &factor);
+    QPointF parallaxFactor() const;
+    QPointF effectiveParallaxFactor() const;
+
     bool canMergeDown() const;
 
     virtual bool isEmpty() const = 0;
@@ -252,17 +257,18 @@ protected:
     Layer *initializeClone(Layer *clone) const;
 
     QString mName;
-    int mId;
+    int mId = 0;
     TypeFlag mLayerType;
-    int mX;
-    int mY;
+    int mX = 0;
+    int mY = 0;
     QPointF mOffset;
-    qreal mOpacity;
+    QPointF mParallaxFactor = { 1.0, 1.0 };
+    qreal mOpacity = 1.0;
     QColor mTintColor;
-    bool mVisible;
-    Map *mMap;
-    GroupLayer *mParentLayer;
-    bool mLocked;
+    bool mVisible = true;
+    Map *mMap = nullptr;
+    GroupLayer *mParentLayer = nullptr;
+    bool mLocked = false;
 
     friend class Map;
     friend class GroupLayer;
@@ -285,6 +291,22 @@ inline QPointF Layer::offset() const
     return mOffset;
 }
 
+/**
+ * Sets the parallax factor of this layer.
+ */
+inline void Layer::setParallaxFactor(const QPointF &factor)
+{
+    mParallaxFactor = factor;
+}
+
+/**
+ * Returns the parallax factor of this layer.
+ */
+inline QPointF Layer::parallaxFactor() const
+{
+    return mParallaxFactor;
+}
+
 
 /**
  * An iterator for iterating over the layers of a map, in the order in which
@@ -294,13 +316,15 @@ inline QPointF Layer::offset() const
  * Modifying the layer hierarchy while an iterator is active will lead to
  * undefined results!
  */
-class TILEDSHARED_EXPORT LayerIterator
+class TILEDSHARED_EXPORT LayerIterator : public std::iterator<std::forward_iterator_tag, Layer*>
 {
 public:
     LayerIterator(const Map *map, int layerTypes = Layer::AnyLayerType);
     LayerIterator(Layer *start);
 
     Layer *currentLayer() const;
+    void setCurrentLayer(Layer *layer);
+
     int currentSiblingIndex() const;
 
     bool hasNextSibling() const;
