@@ -30,7 +30,11 @@
 
 #include "tiled_global.h"
 
+#include <functional>
+#include <memory>
+
 #include <QPainter>
+#include <QPainterPath>
 
 namespace Tiled {
 
@@ -44,7 +48,8 @@ class ImageLayer;
 
 enum RenderFlag {
     ShowTileObjectOutlines = 0x1,
-    ShowTileCollisionShapes = 0x2
+    ShowTileCollisionShapes = 0x2,
+    ShowTileAnimations = 0x4,
 };
 
 Q_DECLARE_FLAGS(RenderFlags, RenderFlag)
@@ -57,11 +62,13 @@ Q_DECLARE_FLAGS(RenderFlags, RenderFlag)
 class TILEDSHARED_EXPORT MapRenderer
 {
 public:
+    enum CellType {
+        OrthogonalCells,
+        HexagonalCells
+    };
+
     MapRenderer(const Map *map)
         : mMap(map)
-        , mFlags(nullptr)
-        , mObjectLineWidth(2)
-        , mPainterScale(1)
     {}
 
     virtual ~MapRenderer();
@@ -128,7 +135,12 @@ public:
      * \a painter.
      */
     virtual void drawGrid(QPainter *painter, const QRectF &rect,
-                          QColor gridColor = Qt::black) const = 0;
+                          QColor gridColor = Qt::black, QSize gridMajor = QSize()) const = 0;
+
+    virtual QPointF snapToGrid(const QPointF &pixelCoords,
+                               int subdivisions = 1) const;
+
+    using RenderTileCallback = std::function<void (QPoint, const QPointF &)>;
 
     /**
      * Draws the given \a layer using the given \a painter.
@@ -136,8 +148,22 @@ public:
      * Optionally, you can pass in the \a exposed rect (of pixels), so that
      * only tiles that can be visible in this area will be drawn.
      */
-    virtual void drawTileLayer(QPainter *painter, const TileLayer *layer,
-                               const QRectF &exposed = QRectF()) const = 0;
+    void drawTileLayer(QPainter *painter, const TileLayer *layer,
+                       const QRectF &exposed = QRectF()) const;
+
+    /**
+     * Calls the given \a renderTile callback for each tile in the given
+     * \a exposed rectangle.
+     *
+     * The callback takes two arguments:
+     *
+     * \list
+     * \li \c tilePos - The tile position of the cell being rendered.
+     * \li \c screenPos - The screen position of the cell being rendered.
+     * \endlist
+     */
+    virtual void drawTileLayer(const RenderTileCallback &renderTile,
+                               const QRectF &exposed) const = 0;
 
     /**
      * Draws the tile selection given by \a region in the specified \a color.
@@ -167,7 +193,7 @@ public:
      */
     void drawImageLayer(QPainter *painter,
                         const ImageLayer *imageLayer,
-                        const QRectF &exposed = QRectF());
+                        const QRectF &exposed = QRectF()) const;
 
     /**
      * Returns the tile coordinates matching the given pixel position.
@@ -244,17 +270,26 @@ public:
     RenderFlags flags() const { return mFlags; }
     void setFlags(RenderFlags flags) { mFlags = flags; }
 
+    CellType cellType() const { return mCellType; }
+
     static QPolygonF lineToPolygon(const QPointF &start, const QPointF &end);
 
+    static std::unique_ptr<MapRenderer> create(const Map *map);
+
 protected:
-    QPen makeGridPen(const QPaintDevice *device, QColor color) const;
+    void setupGridPens(const QPaintDevice *device, QColor color,
+                       QPen &gridPen, QPen &majorGridPen, int gridSize,
+                       QSize gridMajor) const;
+
+    void setCellType(CellType cellType) { mCellType = cellType; }
 
 private:
     const Map *mMap;
 
-    RenderFlags mFlags;
-    qreal mObjectLineWidth;
-    qreal mPainterScale;
+    RenderFlags mFlags = ShowTileAnimations;
+    CellType mCellType = OrthogonalCells;
+    qreal mObjectLineWidth = 2;
+    qreal mPainterScale = 1;
 };
 
 inline const Map *MapRenderer::map() const
@@ -290,21 +325,17 @@ class CellRenderer
 {
 public:
     enum Origin {
-        BottomLeft,
-        BottomCenter
-    };
-
-    enum CellType {
-        OrthogonalCells,
-        HexagonalCells
+        TopLeft,
+        BottomLeft
     };
 
     explicit CellRenderer(QPainter *painter, const MapRenderer *renderer,
-                          CellType cellType = OrthogonalCells);
+                          const QColor &tintColor);
 
     ~CellRenderer() { flush(); }
 
-    void render(const Cell &cell, const QPointF &pos, const QSizeF &size, Origin origin);
+    void render(const Cell &cell, const QPointF &pos, const QSizeF &size,
+                Origin origin = TopLeft);
     void flush();
 
 private:
@@ -315,7 +346,7 @@ private:
     const Tile *mTile;
     QVector<QPainter::PixmapFragment> mFragments;
     const bool mIsOpenGL;
-    const CellType mCellType;
+    const QColor mTintColor;
 };
 
 } // namespace Tiled
