@@ -177,7 +177,7 @@ void MapWriterPrivate::writeObjectTemplate(const ObjectTemplate *objectTemplate,
     mGidMapper.clear();
     if (Tileset *tileset = objectTemplate->object()->cell().tileset()) {
         unsigned firstGid = 1;
-        mGidMapper.insert(firstGid, tileset->sharedPointer());
+        mGidMapper.insert(firstGid, tileset->sharedFromThis());
         writeTileset(writer, *tileset, firstGid);
     }
 
@@ -194,7 +194,7 @@ void MapWriterPrivate::writeMap(QXmlStreamWriter &w, const Map &map)
     const QString orientation = orientationToString(map.orientation());
     const QString renderOrder = renderOrderToString(map.renderOrder());
 
-    w.writeAttribute(QStringLiteral("version"), QLatin1String("1.5"));
+    w.writeAttribute(QStringLiteral("version"), QLatin1String("1.8"));
     w.writeAttribute(QStringLiteral("tiledversion"), QCoreApplication::applicationVersion());
     w.writeAttribute(QStringLiteral("orientation"), orientation);
     w.writeAttribute(QStringLiteral("renderorder"), renderOrder);
@@ -221,6 +221,11 @@ void MapWriterPrivate::writeMap(QXmlStreamWriter &w, const Map &map)
                          staggerIndexToString(map.staggerIndex()));
     }
 
+    if (!map.parallaxOrigin().isNull()) {
+        w.writeAttribute(QStringLiteral("parallaxoriginx"), QString::number(map.parallaxOrigin().x()));
+        w.writeAttribute(QStringLiteral("parallaxoriginy"), QString::number(map.parallaxOrigin().y()));
+    }
+
     if (map.backgroundColor().isValid()) {
         w.writeAttribute(QStringLiteral("backgroundcolor"),
                          colorToString(map.backgroundColor()));
@@ -296,9 +301,8 @@ void MapWriterPrivate::writeTileset(QXmlStreamWriter &w, const Tileset &tileset,
 
         const QString &fileName = tileset.fileName();
         if (!fileName.isEmpty()) {
-            QString source = fileName;
-            if (!mUseAbsolutePaths)
-                source = mDir.relativeFilePath(source);
+            QString source = mUseAbsolutePaths ? fileName
+                                               : filePathRelativeTo(mDir, fileName);
             w.writeAttribute(QStringLiteral("source"), source);
 
             // Tileset is external, so no need to write any of the stuff below
@@ -307,7 +311,7 @@ void MapWriterPrivate::writeTileset(QXmlStreamWriter &w, const Tileset &tileset,
         }
     } else {
         // Include version in external tilesets
-        w.writeAttribute(QStringLiteral("version"), QLatin1String("1.5"));
+        w.writeAttribute(QStringLiteral("version"), QLatin1String("1.8"));
         w.writeAttribute(QStringLiteral("tiledversion"), QCoreApplication::applicationVersion());
     }
 
@@ -388,11 +392,8 @@ void MapWriterPrivate::writeTileset(QXmlStreamWriter &w, const Tileset &tileset,
     const QUrl &imageSource = tileset.imageSource();
     if (!imageSource.isEmpty()) {
         w.writeStartElement(QStringLiteral("image"));
-        QString source;
-        if (mUseAbsolutePaths)
-            source = imageSource.toString(QUrl::PreferLocalFile);
-        else
-            source = toFileReference(imageSource, mDir);
+        QString source = toFileReference(imageSource, mUseAbsolutePaths ? QString()
+                                                                        : mDir.path());
         w.writeAttribute(QStringLiteral("source"), source);
 
         const QColor transColor = tileset.transparentColor();
@@ -445,11 +446,8 @@ void MapWriterPrivate::writeTileset(QXmlStreamWriter &w, const Tileset &tileset,
                     w.writeCharacters(QString::fromLatin1(buffer.data().toBase64()));
                     w.writeEndElement(); // </data>
                 } else {
-                    QString source;
-                    if (mUseAbsolutePaths)
-                        source = tile->imageSource().toString(QUrl::PreferLocalFile);
-                    else
-                        source = toFileReference(tile->imageSource(), mDir);
+                    QString source = toFileReference(tile->imageSource(), mUseAbsolutePaths ? QString()
+                                                                                            : mDir.path());
                     w.writeAttribute(QStringLiteral("source"), source);
                 }
 
@@ -733,7 +731,7 @@ void MapWriterPrivate::writeObject(QXmlStreamWriter &w,
     if (const ObjectTemplate *objectTemplate = mapObject.objectTemplate()) {
         QString fileName = objectTemplate->fileName();
         if (!mUseAbsolutePaths)
-            fileName = mDir.relativeFilePath(fileName);
+            fileName = filePathRelativeTo(mDir, fileName);
         w.writeAttribute(QStringLiteral("template"), fileName);
     }
 
@@ -866,13 +864,18 @@ void MapWriterPrivate::writeImageLayer(QXmlStreamWriter &w,
     w.writeStartElement(QStringLiteral("imagelayer"));
     writeLayerAttributes(w, imageLayer);
 
+    if (imageLayer.repeatX())
+        w.writeAttribute(QStringLiteral("repeatx"), QString::number(imageLayer.repeatX()));
+    if (imageLayer.repeatY())
+        w.writeAttribute(QStringLiteral("repeaty"), QString::number(imageLayer.repeatY()));
+
     // Write the image element
     const QUrl &imageSource = imageLayer.imageSource();
     if (!imageSource.isEmpty()) {
         w.writeStartElement(QStringLiteral("image"));
 
-        QString source = mUseAbsolutePaths ? imageSource.toString(QUrl::PreferLocalFile)
-                                           : toFileReference(imageSource, mDir);
+        QString source = toFileReference(imageSource, mUseAbsolutePaths ? QString()
+                                                                        : mDir.path());
 
         w.writeAttribute(QStringLiteral("source"), source);
 
@@ -916,25 +919,34 @@ void MapWriterPrivate::writeProperties(QXmlStreamWriter &w,
 
     w.writeStartElement(QStringLiteral("properties"));
 
+    const ExportContext context(mUseAbsolutePaths ? QString()
+                                                  : mDir.path());
+
     Properties::const_iterator it = properties.constBegin();
     Properties::const_iterator it_end = properties.constEnd();
     for (; it != it_end; ++it) {
         w.writeStartElement(QStringLiteral("property"));
         w.writeAttribute(QStringLiteral("name"), it.key());
 
-        int type = it.value().userType();
-        QString typeName = typeToName(type);
-        if (typeName != QLatin1String("string"))
-            w.writeAttribute(QStringLiteral("type"), typeName);
+        const auto exportValue = context.toExportValue(it.value());
+        if (exportValue.typeName != QLatin1String("string"))
+            w.writeAttribute(QStringLiteral("type"), exportValue.typeName);
+        if (!exportValue.propertyTypeName.isEmpty())
+            w.writeAttribute(QStringLiteral("propertytype"), exportValue.propertyTypeName);
 
-        QVariant exportValue = mUseAbsolutePaths ? toExportValue(it.value())
-                                                 : toExportValue(it.value(), mDir);
-        QString value = exportValue.toString();
+        // For class property values, write out the original value, so that the
+        // propertytype attribute can also be written for their members where
+        // applicable.
+        if (exportValue.value.userType() == QMetaType::QVariantMap) {
+            writeProperties(w, it.value().value<PropertyValue>().value.toMap());
+        } else {
+            const QString value = exportValue.value.toString();
 
-        if (value.contains(QLatin1Char('\n')))
-            w.writeCharacters(value);
-        else
-            w.writeAttribute(QStringLiteral("value"), value);
+            if (value.contains(QLatin1Char('\n')))
+                w.writeCharacters(value);
+            else
+                w.writeAttribute(QStringLiteral("value"), value);
+        }
 
         w.writeEndElement();
     }
