@@ -63,7 +63,7 @@ void ObjectTemplate::setObject(const MapObject *object)
     }
 
     if (tileset)
-        mTileset = tileset->sharedPointer();
+        mTileset = tileset->sharedFromThis();
     else
         mTileset.reset();
 }
@@ -75,7 +75,7 @@ void ObjectTemplate::setObject(std::unique_ptr<MapObject> object)
 
     Tileset *tileset = mObject->cell().tileset();
     if (tileset)
-        mTileset = tileset->sharedPointer();
+        mTileset = tileset->sharedFromThis();
     else
         mTileset.reset();
 }
