@@ -30,12 +30,13 @@
 
 #include "tiled_global.h"
 
+#include <QColor>
+#include <QDir>
+#include <QMetaType>
 #include <QRectF>
 #include <QString>
 #include <QUrl>
 
-class QDir;
-
 namespace Tiled {
 
 enum FlipDirection {
@@ -49,6 +50,7 @@ enum RotateDirection {
 };
 
 enum Alignment {
+    Unspecified,
     TopLeft,
     Top,
     TopRight,
@@ -68,19 +70,51 @@ enum LoadingStatus {
 };
 
 const int CHUNK_SIZE = 16;
+const int CHUNK_BITS = 4;
 const int CHUNK_SIZE_MIN = 4;
 const int CHUNK_MASK = CHUNK_SIZE - 1;
 
 static const char TILES_MIMETYPE[] = "application/vnd.tile.list";
 static const char FRAMES_MIMETYPE[] = "application/vnd.frame.list";
 static const char LAYERS_MIMETYPE[] = "application/vnd.layer.list";
-static const char TEMPLATES_MIMETYPE[] = "application/vnd.templates.list";
 static const char PROPERTIES_MIMETYPE[] = "application/vnd.properties.list";
 
 TILEDSHARED_EXPORT QPointF alignmentOffset(const QRectF &r, Alignment alignment);
 
-TILEDSHARED_EXPORT QString toFileReference(const QUrl &url, const QDir &dir);
-TILEDSHARED_EXPORT QUrl toUrl(const QString &reference, const QDir &dir);
+TILEDSHARED_EXPORT QString toFileReference(const QUrl &url, const QString &path = QString());
+TILEDSHARED_EXPORT QUrl toUrl(const QString &filePathOrUrl, const QString &path = QString());
 TILEDSHARED_EXPORT QString urlToLocalFileOrQrc(const QUrl &url);
+TILEDSHARED_EXPORT QString filePathRelativeTo(const QDir &dir, const QString &filePath);
+
+inline QString toFileReference(const QUrl &url, const QDir &dir)
+{ return toFileReference(url, dir.path()); }
+
+inline QUrl toUrl(const QString &filePathOrUrl, const QDir &dir)
+{ return toUrl(filePathOrUrl, dir.path()); }
+
+inline QString colorToString(const QColor &color)
+{
+    if (color.alpha() != 255)
+        return color.name(QColor::HexArgb);
+    return color.name();
+}
+
+inline QMargins maxMargins(const QMargins &a,
+                           const QMargins &b)
+{
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    return QMargins(qMax(a.left(), b.left()),
+                    qMax(a.top(), b.top()),
+                    qMax(a.right(), b.right()),
+                    qMax(a.bottom(), b.bottom()));
+#else
+    return a | b;
+#endif
+}
+
+TILEDSHARED_EXPORT QString alignmentToString(Alignment);
+TILEDSHARED_EXPORT Alignment alignmentFromString(const QString &);
 
 } // namespace Tiled
+
+Q_DECLARE_METATYPE(Tiled::Alignment);
