@@ -45,12 +45,14 @@
 
 #include <functional>
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 inline uint qHash(QPoint key, uint seed = 0) Q_DECL_NOTHROW
 {
     uint h1 = qHash(key.x(), seed);
     uint h2 = qHash(key.y(), seed);
     return ((h1 << 16) | (h1 >> 16)) ^ h2 ^ seed;
 }
+#endif
 
 namespace Tiled {
 
@@ -118,6 +120,8 @@ public:
     void setFlippedAntiDiagonally(bool v) { v ? _flags |= FlippedAntiDiagonally : _flags &= ~FlippedAntiDiagonally; }
     void setRotatedHexagonal120(bool v) { v ? _flags |= RotatedHexagonal120 : _flags &= ~RotatedHexagonal120; }
 
+    void rotate(RotateDirection direction);
+
     bool checked() const { return _flags & Checked; }
     void setChecked(bool checked) { checked ? _flags |= Checked : _flags &= ~Checked; }
 
@@ -603,15 +607,13 @@ inline bool TileLayer::contains(QPoint point) const
 
 inline Chunk& TileLayer::chunk(int x, int y)
 {
-    QPoint chunkCoordinates(x < 0 ? (x + 1) / CHUNK_SIZE - 1 : x / CHUNK_SIZE,
-                            y < 0 ? (y + 1) / CHUNK_SIZE - 1 : y / CHUNK_SIZE);
+    const QPoint chunkCoordinates(x >> CHUNK_BITS, y >> CHUNK_BITS);
     return mChunks[chunkCoordinates];
 }
 
 inline const Chunk* TileLayer::findChunk(int x, int y) const
 {
-    QPoint chunkCoordinates(x < 0 ? (x + 1) / CHUNK_SIZE - 1 : x / CHUNK_SIZE,
-                            y < 0 ? (y + 1) / CHUNK_SIZE - 1 : y / CHUNK_SIZE);
+    const QPoint chunkCoordinates(x >> CHUNK_BITS, y >> CHUNK_BITS);
     auto it = mChunks.find(chunkCoordinates);
     return it != mChunks.end() ? &it.value() : nullptr;
 }
@@ -648,7 +650,7 @@ inline void TileLayer::setCells(int x, int y, const TileLayer *tileLayer)
              QRect(x, y, tileLayer->width(), tileLayer->height()));
 }
 
-typedef QSharedPointer<TileLayer> SharedTileLayer;
+using SharedTileLayer = QSharedPointer<TileLayer>;
 
 } // namespace Tiled
 
