@@ -200,18 +200,17 @@ std::unique_ptr<World> WorldManager::privateLoadWorld(const QString &fileName,
 
 World *WorldManager::addEmptyWorld(const QString &fileName, QString *errorString)
 {
-    World *world = new World();
+    std::unique_ptr<World> world { new World() };
     world->fileName = fileName;
     world->onlyShowAdjacentMaps = false;
 
     if (mWorlds.contains(fileName)) {
-        if (errorString) {
+        if (errorString)
             *errorString = QLatin1String("World already loaded");
-        }
         return nullptr;
     }
 
-    mWorlds.insert(fileName, world);
+    mWorlds.insert(fileName, world.release());
 
     if (saveWorld(fileName, errorString)) {
         emit worldsChanged();
@@ -493,8 +492,13 @@ QRect World::mapRect(const QString &fileName) const
     for (const World::Pattern &pattern : patterns) {
         QRegularExpressionMatch match = pattern.regexp.match(fileName);
         if (match.hasMatch()) {
+#if QT_VERSION >= QT_VERSION_CHECK(6,0,0)
+            const int x = match.capturedView(1).toInt();
+            const int y = match.capturedView(2).toInt();
+#else
             const int x = match.capturedRef(1).toInt();
             const int y = match.capturedRef(2).toInt();
+#endif
 
             return QRect(QPoint(x * pattern.multiplierX,
                                 y * pattern.multiplierY) + pattern.offset,
@@ -517,8 +521,13 @@ QVector<World::MapEntry> World::allMaps() const
             for (const QString &fileName : entries) {
                 QRegularExpressionMatch match = pattern.regexp.match(fileName);
                 if (match.hasMatch()) {
+#if QT_VERSION >= QT_VERSION_CHECK(6,0,0)
+                    const int x = match.capturedView(1).toInt();
+                    const int y = match.capturedView(2).toInt();
+#else
                     const int x = match.capturedRef(1).toInt();
                     const int y = match.capturedRef(2).toInt();
+#endif
 
                     MapEntry entry;
                     entry.fileName = dir.filePath(fileName);
@@ -605,3 +614,5 @@ QString World::displayName(const QString &fileName)
 }
 
 } // namespace Tiled
+
+#include "moc_worldmanager.cpp"
