@@ -31,10 +31,10 @@
 #include "objecttemplate.h"
 #include "properties.h"
 #include "savefile.h"
-#include "terrain.h"
 #include "tile.h"
 #include "tilelayer.h"
 #include "tileset.h"
+#include "wangset.h"
 
 #include <QCoreApplication>
 #include <QDir>
@@ -67,6 +67,7 @@ public:
     void writeTileset(const Tiled::Tileset &,
                       unsigned firstGid,
                       bool embedded = true);
+    void writeWangSet(const Tiled::WangSet &);
     void writeLayers(const QList<Tiled::Layer*> &layers,
                      Tiled::Map::LayerDataFormat format,
                      int compressionLevel,
@@ -206,7 +207,7 @@ void LuaWriter::writeMap(const Map *map)
     mWriter.writeStartDocument();
     mWriter.writeStartReturnTable();
 
-    mWriter.writeKeyAndValue("version", "1.4");
+    mWriter.writeKeyAndValue("version", "1.5");
     mWriter.writeKeyAndValue("luaversion", "5.1");
     mWriter.writeKeyAndValue("tiledversion", QCoreApplication::applicationVersion());
 
@@ -232,6 +233,13 @@ void LuaWriter::writeMap(const Map *map)
                                  staggerIndexToString(map->staggerIndex()));
     }
 
+    if (!map->parallaxOrigin().isNull()) {
+        mWriter.writeStartTable("parallaxorigin");
+        mWriter.writeKeyAndValue("x", map->parallaxOrigin().x());
+        mWriter.writeKeyAndValue("y", map->parallaxOrigin().y());
+        mWriter.writeEndTable();
+    }
+
     const QColor &backgroundColor = map->backgroundColor();
     if (backgroundColor.isValid())
         writeColor("backgroundcolor", backgroundColor);
@@ -259,18 +267,20 @@ void LuaWriter::writeProperties(const Properties &properties)
 {
     mWriter.writeStartTable("properties");
 
+    const ExportContext context(mDir.path());
+
     Properties::const_iterator it = properties.constBegin();
     Properties::const_iterator it_end = properties.constEnd();
     for (; it != it_end; ++it) {
+        const auto exportValue = context.toExportValue(it.value());
         if (it.value().userType() == objectRefTypeId()) {
             mWriter.writeStartTable(it.key());
             mWriter.setSuppressNewlines(true);
-            mWriter.writeKeyAndValue("id", it.value().value<ObjectRef>().id);
+            mWriter.writeKeyAndValue("id", exportValue.value.toInt());
             mWriter.writeEndTable();
             mWriter.setSuppressNewlines(false);
         } else {
-            const QVariant value = toExportValue(it.value(), mDir);
-            mWriter.writeQuotedKeyAndValue(it.key(), value);
+            mWriter.writeQuotedKeyAndValue(it.key(), exportValue.value);
         }
     }
 
@@ -289,8 +299,6 @@ static bool includeTile(const Tile *tile)
         return true;
     if (tile->isAnimated())
         return true;
-    if (tile->terrain() != 0xFFFFFFFF)
-        return true;
     if (tile->probability() != 1.0)
         return true;
 
@@ -307,18 +315,31 @@ void LuaWriter::writeTileset(const Tileset &tileset,
         mWriter.writeStartReturnTable();
 
         // Include version in external tilesets
-        mWriter.writeKeyAndValue("version", "1.4");
+        mWriter.writeKeyAndValue("version", "1.5");
         mWriter.writeKeyAndValue("luaversion", "5.1");
         mWriter.writeKeyAndValue("tiledversion", QCoreApplication::applicationVersion());
     }
 
     mWriter.writeKeyAndValue("name", tileset.name());
-    if (embedded)
+    if (embedded) {
         mWriter.writeKeyAndValue("firstgid", firstGid);
 
-    if (!tileset.fileName().isEmpty() && embedded) {
-        const QString rel = mDir.relativeFilePath(tileset.fileName());
-        mWriter.writeKeyAndValue("filename", rel);
+        if (tileset.isExternal()) {
+            const QString rel = filePathRelativeTo(mDir, tileset.fileName());
+            mWriter.writeKeyAndValue("filename", rel);
+
+            // For those exporting their tilesets separately, it could be
+            // helpful to include a relative reference to the exported file as
+            // well. For consistency I decided to include this separately from
+            // the "filename".
+            if (!tileset.exportFileName.isEmpty()) {
+                const QString rel = filePathRelativeTo(mDir, tileset.exportFileName);
+                mWriter.writeKeyAndValue("exportfilename", rel);
+            }
+
+            mWriter.writeEndTable(); // tileset
+            return;
+        }
     }
 
     /* Include all tileset information even for external tilesets, since the
@@ -363,25 +384,19 @@ void LuaWriter::writeTileset(const Tileset &tileset,
 
     writeProperties(tileset.properties());
 
-    mWriter.writeStartTable("terrains");
-    for (int i = 0; i < tileset.terrainCount(); ++i) {
-        const Terrain *t = tileset.terrain(i);
-        mWriter.writeStartTable();
-
-        mWriter.writeKeyAndValue("name", t->name());
-        mWriter.writeKeyAndValue("tile", t->imageTileId());
-
-        writeProperties(t->properties());
-
-        mWriter.writeEndTable();
-    }
+    mWriter.writeStartTable("wangsets");
+    for (int i = 0; i < tileset.wangSetCount(); ++i)
+        writeWangSet(*tileset.wangSet(i));
     mWriter.writeEndTable();
 
     mWriter.writeKeyAndValue("tilecount", tileset.tileCount());
     mWriter.writeStartTable("tiles");
+
+    const bool includeAllTiles = tileset.anyTileOutOfOrder();
+
     for (const Tile *tile : tileset.tiles()) {
         // For brevity only write tiles with interesting properties
-        if (!includeTile(tile))
+        if (!includeAllTiles && !includeTile(tile))
             continue;
 
         mWriter.writeStartTable();
@@ -403,16 +418,6 @@ void LuaWriter::writeTileset(const Tileset &tileset,
             }
         }
 
-        unsigned terrain = tile->terrain();
-        if (terrain != 0xFFFFFFFF) {
-            mWriter.writeStartTable("terrain");
-            mWriter.setSuppressNewlines(true);
-            for (int i = 0; i < 4; ++i )
-                mWriter.writeValue(tile->cornerTerrainId(i));
-            mWriter.writeEndTable();
-            mWriter.setSuppressNewlines(false);
-        }
-
         if (tile->probability() != 1.0)
             mWriter.writeKeyAndValue("probability", tile->probability());
 
@@ -442,6 +447,52 @@ void LuaWriter::writeTileset(const Tileset &tileset,
         mWriter.writeEndDocument();
 }
 
+void LuaWriter::writeWangSet(const WangSet &wangSet)
+{
+    mWriter.writeStartTable();
+
+    mWriter.writeKeyAndValue("name", wangSet.name());
+    mWriter.writeKeyAndValue("tile", wangSet.imageTileId());
+
+    writeProperties(wangSet.properties());
+
+    mWriter.writeStartTable("colors");
+    for (int i = 1; i <= wangSet.colorCount(); ++i) {
+        const WangColor &wangColor = *wangSet.colorAt(i);
+        mWriter.writeStartTable();
+
+        writeColor("color", wangColor.color());
+        mWriter.writeKeyAndValue("name", wangColor.name());
+        mWriter.writeKeyAndValue("probability", wangColor.probability());
+        mWriter.writeKeyAndValue("tile", wangColor.imageId());
+
+        writeProperties(wangColor.properties());
+
+        mWriter.writeEndTable();
+    }
+    mWriter.writeEndTable();
+
+    mWriter.writeStartTable("wangtiles");
+    const auto wangTiles = wangSet.sortedWangTiles();
+    for (const WangTile &wangTile : wangTiles) {
+        mWriter.writeStartTable();
+
+        mWriter.writeStartTable("wangid");
+        mWriter.setSuppressNewlines(true);
+        for (int i = 0; i < WangId::NumIndexes; ++i)
+            mWriter.writeValue(wangTile.wangId().indexColor(i));
+        mWriter.writeEndTable();
+        mWriter.setSuppressNewlines(false);
+
+        mWriter.writeKeyAndValue("tileid", wangTile.tileId());
+
+        mWriter.writeEndTable();
+    }
+    mWriter.writeEndTable();
+
+    mWriter.writeEndTable();
+}
+
 void LuaWriter::writeLayers(const QList<Layer *> &layers,
                             Map::LayerDataFormat format,
                             int compressionLevel,
@@ -602,6 +653,9 @@ void LuaWriter::writeImageLayer(const ImageLayer *imageLayer)
     }
 
     writeLayerProperties(imageLayer);
+    mWriter.writeKeyAndValue("repeatx", imageLayer->repeatX());
+    mWriter.writeKeyAndValue("repeaty", imageLayer->repeatY());
+
     writeProperties(imageLayer->properties());
 
     mWriter.writeEndTable();
@@ -700,6 +754,10 @@ void LuaWriter::writeLayerProperties(const Layer *layer)
     mWriter.writeKeyAndValue("offsetx", offset.x());
     mWriter.writeKeyAndValue("offsety", offset.y());
 
+    const QPointF parallaxFactor = layer->parallaxFactor();
+    mWriter.writeKeyAndValue("parallaxx", parallaxFactor.x());
+    mWriter.writeKeyAndValue("parallaxy", parallaxFactor.y());
+
     if (layer->tintColor().isValid())
         writeColor("tintcolor", layer->tintColor());
 }
