@@ -52,6 +52,7 @@
 #include <QStyle>
 #include <QPalette>
 #include <QScreen>
+#include <QScrollBar>
 
 #ifndef Q_OS_MAC
 static qreal defaultDpiScale()
@@ -174,6 +175,7 @@ QtPropertyEditorView::QtPropertyEditorView(QWidget *parent) :
     QTreeWidget(parent),
     m_editorPrivate(0)
 {
+    setUniformRowHeights(true);
     connect(header(), SIGNAL(sectionDoubleClicked(int)), this, SLOT(resizeColumnToContents(int)));
 }
 
@@ -500,7 +502,7 @@ static QIcon drawIndicatorIcon(const QPalette &palette, QStyle *style)
 void QtTreePropertyBrowserPrivate::init(QWidget *parent)
 {
     QHBoxLayout *layout = new QHBoxLayout(parent);
-    layout->setMargin(0);
+    layout->setContentsMargins(0, 0, 0, 0);
     m_treeWidget = new QtPropertyEditorView(parent);
     m_treeWidget->setEditorPrivate(this);
     m_treeWidget->setIconSize(QSize(18, 18));
@@ -1005,6 +1007,16 @@ void QtTreePropertyBrowser::setSplitterPosition(int position)
     d_ptr->m_treeWidget->header()->resizeSection(0, position);
 }
 
+int QtTreePropertyBrowser::scrollPosition() const
+{
+    return d_ptr->m_treeWidget->verticalScrollBar()->value();
+}
+
+void QtTreePropertyBrowser::setScrollPosition(int value)
+{
+    d_ptr->m_treeWidget->verticalScrollBar()->setValue(value);
+}
+
 /*!
     Sets the \a item to either collapse or expanded, depending on the value of \a expanded.
 
