@@ -21,6 +21,7 @@
 #pragma once
 
 #include "abstracttool.h"
+#include "preferences.h"
 
 class QAction;
 
@@ -38,9 +39,17 @@ class MapObjectItem;
 class AbstractObjectTool : public AbstractTool
 {
     Q_OBJECT
-    Q_INTERFACES(Tiled::AbstractTool)
 
 public:
+    enum SelectionBehavior {
+        AllLayers,
+        PreferSelectedLayers,
+        SelectedLayers
+    };
+    Q_ENUM(SelectionBehavior)
+
+    static Preference<SelectionBehavior> ourSelectionBehavior;
+
     /**
      * Constructs an abstract object tool with the given \a name and \a icon.
      */
@@ -50,9 +59,6 @@ public:
                        const QKeySequence &shortcut,
                        QObject *parent = nullptr);
 
-    void activate(MapScene *scene) override;
-    void deactivate(MapScene *scene) override;
-
     void keyPressed(QKeyEvent *event) override;
     void mouseLeft() override;
     void mouseMoved(const QPointF &pos, Qt::KeyboardModifiers modifiers) override;
@@ -62,6 +68,9 @@ public:
 
     void populateToolBar(QToolBar*) override;
 
+    static SelectionBehavior selectionBehavior();
+    void filterMapObjects(QList<MapObject*> &mapObjects) const;
+
 protected:
     /**
      * Overridden to only enable this tool when the currently selected layer is
@@ -69,26 +78,27 @@ protected:
      */
     void updateEnabledState() override;
 
-    MapScene *mapScene() const { return mMapScene; }
     ObjectGroup *currentObjectGroup() const;
     QList<MapObject*> mapObjectsAt(const QPointF &pos) const;
     MapObject *topMostMapObjectAt(const QPointF &pos) const;
 
+    virtual void flipHorizontally();
+    virtual void flipVertically();
+    virtual void rotateLeft();
+    virtual void rotateRight();
+
 private:
     void duplicateObjects();
     void removeObjects();
+    void applyCollisionsToSelectedTiles(bool replace);
     void resetTileSize();
+    void convertRectanglesToPolygons();
     void saveSelectedObject();
     void detachSelectedObjects();
     void replaceObjectsWithTemplate();
     void resetInstances();
     void changeTile();
 
-    void flipHorizontally();
-    void flipVertically();
-    void rotateLeft();
-    void rotateRight();
-
     void raise();
     void lower();
     void raiseToTop();
@@ -97,8 +107,6 @@ private:
     void showContextMenu(MapObject *clickedObject,
                          QPoint screenPos);
 
-    MapScene *mMapScene;
-
     QAction *mFlipHorizontal;
     QAction *mFlipVertical;
     QAction *mRotateLeft;
