@@ -115,6 +115,16 @@ ObjectTemplate *AbstractTool::objectTemplate() const
     return toolManager()->objectTemplate();
 }
 
+void AbstractTool::activate(MapScene *scene)
+{
+    mMapScene = scene;
+}
+
+void AbstractTool::deactivate(MapScene *)
+{
+    mMapScene = nullptr;
+}
+
 void AbstractTool::keyPressed(QKeyEvent *event)
 {
     event->ignore();
@@ -133,8 +143,6 @@ void AbstractTool::setMapDocument(MapDocument *mapDocument)
     if (mMapDocument) {
         disconnect(mMapDocument, &MapDocument::changed,
                    this, &AbstractTool::changeEvent);
-        disconnect(mMapDocument, &MapDocument::currentLayerChanged,
-                   this, &AbstractTool::updateEnabledState);
     }
 
     MapDocument *oldDocument = mMapDocument;
@@ -144,10 +152,7 @@ void AbstractTool::setMapDocument(MapDocument *mapDocument)
     if (mMapDocument) {
         connect(mMapDocument, &MapDocument::changed,
                 this, &AbstractTool::changeEvent);
-        connect(mMapDocument, &MapDocument::currentLayerChanged,
-                this, &AbstractTool::updateEnabledState);
     }
-    updateEnabledState();
 }
 
 void AbstractTool::changeEvent(const ChangeEvent &event)
@@ -174,3 +179,5 @@ Layer *AbstractTool::currentLayer() const
 }
 
 } // namespace Tiled
+
+#include "moc_abstracttool.cpp"
