@@ -53,7 +53,7 @@ class ActionManager : public QObject
     Q_OBJECT
 
     explicit ActionManager(QObject *parent = nullptr);
-    ~ActionManager();
+    ~ActionManager() override;
 
 public:
     struct MenuItem {
