@@ -23,7 +23,10 @@
 #include "ui_addpropertydialog.h"
 
 #include "documentmanager.h"
+#include "object.h"
+#include "preferences.h"
 #include "properties.h"
+#include "propertytypesmodel.h"
 #include "session.h"
 #include "utils.h"
 
@@ -38,6 +41,18 @@ static SessionOption<QString> propertyType { "property.type", QStringLiteral("st
 AddPropertyDialog::AddPropertyDialog(QWidget *parent)
     : QDialog(parent)
     , mUi(new Ui::AddPropertyDialog)
+{
+    initialize(nullptr);
+}
+
+AddPropertyDialog::AddPropertyDialog(const ClassPropertyType *parentClassType, QWidget *parent)
+    : QDialog(parent)
+    , mUi(new Ui::AddPropertyDialog)
+{
+    initialize(parentClassType);
+}
+
+void AddPropertyDialog::initialize(const Tiled::ClassPropertyType *parentClassType)
 {
 #if QT_VERSION < QT_VERSION_CHECK(5, 10, 0)
     setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
@@ -46,14 +61,26 @@ AddPropertyDialog::AddPropertyDialog(QWidget *parent)
     mUi->setupUi(this);
     resize(Utils::dpiScaled(size()));
 
+    const QIcon plain(QStringLiteral("://images/scalable/property-type-plain.svg"));
+
     // Add possible types from QVariant
-    mUi->typeBox->addItem(typeToName(QVariant::Bool),    false);
-    mUi->typeBox->addItem(typeToName(QVariant::Color),   QColor());
-    mUi->typeBox->addItem(typeToName(QVariant::Double),  0.0);
-    mUi->typeBox->addItem(typeToName(filePathTypeId()),  QVariant::fromValue(FilePath()));
-    mUi->typeBox->addItem(typeToName(QVariant::Int),     0);
-    mUi->typeBox->addItem(typeToName(objectRefTypeId()), QVariant::fromValue(ObjectRef()));
-    mUi->typeBox->addItem(typeToName(QVariant::String),  QString());
+    mUi->typeBox->addItem(plain, typeToName(QMetaType::Bool),      false);
+    mUi->typeBox->addItem(plain, typeToName(QMetaType::QColor),    QColor());
+    mUi->typeBox->addItem(plain, typeToName(QMetaType::Double),    0.0);
+    mUi->typeBox->addItem(plain, typeToName(filePathTypeId()),     QVariant::fromValue(FilePath()));
+    mUi->typeBox->addItem(plain, typeToName(QMetaType::Int),       0);
+    mUi->typeBox->addItem(plain, typeToName(objectRefTypeId()),    QVariant::fromValue(ObjectRef()));
+    mUi->typeBox->addItem(plain, typeToName(QMetaType::QString),   QString());
+
+    for (const auto propertyType : Object::propertyTypes()) {
+        // Avoid suggesting the creation of circular dependencies between types
+        if (parentClassType && !parentClassType->canAddMemberOfType(propertyType))
+            continue;
+
+        const QVariant var = propertyType->wrap(propertyType->defaultValue());
+        const QIcon icon = PropertyTypesModel::iconForPropertyType(propertyType->type);
+        mUi->typeBox->addItem(icon, propertyType->name, var);
+    }
 
     mUi->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
 
@@ -90,3 +117,5 @@ void AddPropertyDialog::typeChanged(const QString &text)
 {
     session::propertyType = text;
 }
+
+#include "moc_addpropertydialog.cpp"
