@@ -44,11 +44,13 @@ TileStamp CaptureStampHelper::endCapture(const MapDocument &mapDocument, QPoint
     mActive = false;
 
     QRect captured = capturedArea(tilePosition);
-    std::unique_ptr<Map> stamp { new Map(mapDocument.map()->orientation(),
-                                         captured.width(),
-                                         captured.height(),
-                                         mapDocument.map()->tileWidth(),
-                                         mapDocument.map()->tileHeight()) };
+
+    Map::Parameters mapParameters = mapDocument.map()->parameters();
+    mapParameters.width = captured.width();
+    mapParameters.height = captured.height();
+    mapParameters.infinite = false;
+
+    auto stamp = std::make_unique<Map>(mapParameters);
 
     // Iterate all layers to make sure we're adding layers in the right order
     LayerIterator it(mapDocument.map(), Layer::TileLayerType);
@@ -58,30 +60,24 @@ TileStamp CaptureStampHelper::endCapture(const MapDocument &mapDocument, QPoint
 
         // Intersect with the layer and translate to layer coordinates
         QRect capturedFromLayer = captured.intersected(tileLayer->bounds());
-        if (!captured.isValid())
-            continue;
         capturedFromLayer.translate(-tileLayer->position());
 
         auto capture = tileLayer->copy(capturedFromLayer);
         capture->setName(tileLayer->name());
         capture->setPosition(capturedFromLayer.topLeft() - captured.topLeft());
+        capture->setOpacity(tileLayer->opacity());
+        capture->setTintColor(tileLayer->tintColor());
 
         stamp->addLayer(std::move(capture));
     }
 
     if (stamp->layerCount() > 0) {
-        auto staggerAxis = mapDocument.map()->staggerAxis();
-        auto staggerIndex = mapDocument.map()->staggerIndex();
+        // Adjust the stagger index when the capture starts at an odd offset
+        const auto staggerIndex = stamp->staggerIndex();
+        const int staggerOffSet = (stamp->staggerAxis() == Map::StaggerX ? captured.x()
+                                                                         : captured.y()) % 2;
 
-        // Gets if the relative stagger should be the same as the base layer
-        int staggerIndexOffSet;
-        if (staggerAxis == Map::StaggerX)
-            staggerIndexOffSet = captured.x() % 2;
-        else
-            staggerIndexOffSet = captured.y() % 2;
-
-        stamp->setStaggerAxis(staggerAxis);
-        stamp->setStaggerIndex(static_cast<Map::StaggerIndex>((staggerIndex + staggerIndexOffSet) % 2));
+        stamp->setStaggerIndex(static_cast<Map::StaggerIndex>((staggerIndex + staggerOffSet) % 2));
 
         // Add tileset references to map
         stamp->addTilesets(stamp->usedTilesets());
