@@ -26,6 +26,7 @@
 
 #include <QCoreApplication>
 
+#include "addremovetileset.h"
 #include "changeevents.h"
 #include "qtcompat_p.h"
 
@@ -137,6 +138,14 @@ ChangeMapObjectsTile::ChangeMapObjectsTile(Document *document,
 
         mOldChangedProperties.append(object->changedProperties());
     }
+
+    // Make sure the tileset is part of the document
+    if (tile && document->type() == Document::MapDocumentType) {
+        auto mapDocument = static_cast<MapDocument*>(document);
+        SharedTileset sharedTileset = tile->sharedTileset();
+        if (!mapDocument->map()->tilesets().contains(sharedTileset))
+            new AddTileset(mapDocument, sharedTileset, this);
+    }
 }
 
 static void setObjectCell(MapObject *object,
