@@ -22,13 +22,13 @@
 
 #include "mapobject.h"
 #include "tilelayer.h"
+#include "undocommands.h"
 
 #include <QUndoCommand>
 #include <QVector>
 
 namespace Tiled {
 
-class MapObject;
 class ObjectTemplate;
 class Tile;
 
@@ -49,6 +49,10 @@ public:
     void undo() override { swap(); }
     void redo() override { swap(); }
 
+    int id() const override { return Cmd_ChangeMapObject; }
+
+    bool mergeWith(const QUndoCommand *other) final;
+
 private:
     void swap();
 
@@ -107,7 +111,7 @@ private:
     Tile * const mTile;
     QVector<Cell> mOldCells;
     QVector<bool> mUpdateSize;
-    QVector<bool> mOldChangeStates;
+    QVector<MapObject::ChangedProperties> mOldChangedProperties;
 };
 
 class DetachObjects : public QUndoCommand
