@@ -20,6 +20,8 @@
 
 #pragma once
 
+#include "undocommands.h"
+
 #include <QRegion>
 #include <QUndoCommand>
 
@@ -41,6 +43,8 @@ public:
     void undo() override;
     void redo() override;
 
+    int id() const override { return Cmd_ChangeSelectedArea; }
+
 private:
     void swapSelection();
 
