@@ -36,8 +36,8 @@ public:
                           Tile *tile,
                           const QUrl &imageSource);
 
-    void undo() { apply(mOldImageSource); }
-    void redo() { apply(mNewImageSource); }
+    void undo() override { apply(mOldImageSource); }
+    void redo() override { apply(mNewImageSource); }
 
 private:
     void apply(const QUrl &imageSource);
