@@ -24,17 +24,20 @@
 
 namespace Tiled {
 
-namespace Internal {
-
 class CreateEllipseObjectTool : public CreateScalableObjectTool
 {
     Q_OBJECT
+
 public:
     CreateEllipseObjectTool(QObject *parent);
+
     void languageChanged() override;
+
 protected:
     MapObject *createNewMapObject() override;
+
+private:
+    void languageChangedImpl();
 };
 
-}
-}
+} // namespace Tiled
