@@ -1083,18 +1083,18 @@ void DocumentManager::tilesetAdded(int index, Tileset *tileset)
 {
     Q_UNUSED(index)
     MapDocument *mapDocument = static_cast<MapDocument*>(QObject::sender());
-    addToTilesetDocument(tileset->sharedPointer(), mapDocument);
+    addToTilesetDocument(tileset->sharedFromThis(), mapDocument);
 }
 
 void DocumentManager::tilesetRemoved(Tileset *tileset)
 {
     MapDocument *mapDocument = static_cast<MapDocument*>(QObject::sender());
-    removeFromTilesetDocument(tileset->sharedPointer(), mapDocument);
+    removeFromTilesetDocument(tileset->sharedFromThis(), mapDocument);
 }
 
 void DocumentManager::tilesetNameChanged(Tileset *tileset)
 {
-    auto *tilesetDocument = findTilesetDocument(tileset->sharedPointer());
+    auto *tilesetDocument = findTilesetDocument(tileset->sharedFromThis());
     if (tilesetDocument->isEmbedded())
         updateDocumentTab(tilesetDocument);
 }
@@ -1164,9 +1164,9 @@ TilesetDocument* DocumentManager::findTilesetDocument(const QString &fileName) c
 }
 
 /**
- * Opens the document for the given \a tileset.
+ * Opens the document for the given \a tileset. Returns the tileset's document.
  */
-void DocumentManager::openTileset(const SharedTileset &tileset)
+TilesetDocument *DocumentManager::openTileset(const SharedTileset &tileset)
 {
     TilesetDocumentPtr tilesetDocument;
     if (auto existingTilesetDocument = findTilesetDocument(tileset))
@@ -1176,6 +1176,8 @@ void DocumentManager::openTileset(const SharedTileset &tileset)
 
     if (!switchToDocument(tilesetDocument.data()))
         addDocument(tilesetDocument);
+
+    return tilesetDocument.data();
 }
 
 void DocumentManager::addToTilesetDocument(const SharedTileset &tileset, MapDocument *mapDocument)
@@ -1319,7 +1321,7 @@ void DocumentManager::tilesetImagesChanged(Tileset *tileset)
     if (!mayNeedColumnCountAdjustment(*tileset))
         return;
 
-    SharedTileset sharedTileset = tileset->sharedPointer();
+    SharedTileset sharedTileset = tileset->sharedFromThis();
     QList<Document*> affectedDocuments;
 
     for (const auto &document : qAsConst(mDocuments)) {
