@@ -23,6 +23,7 @@
 #include "editableobject.h"
 
 #include <QJSValue>
+#include <QSharedPointer>
 
 #include <memory>
 
@@ -46,7 +47,7 @@ public:
     EditableAsset(Document *document, Object *object, QObject *parent = nullptr);
 
     QString fileName() const;
-    virtual bool isReadOnly() const = 0;
+    bool isReadOnly() const override = 0;
     bool isMap() const;
     bool isTileset() const;
 
@@ -59,6 +60,11 @@ public:
 
     Document *document() const;
 
+    /**
+     * Creates a document for this asset.
+     */
+    virtual QSharedPointer<Document> createDocument() = 0;
+
 public slots:
     void undo();
     void redo();
@@ -68,7 +74,10 @@ signals:
     void fileNameChanged(const QString &fileName, const QString &oldFileName);
 
 private:
-    Document * const mDocument;
+    friend class Document;
+    void setDocument(Document *document);
+
+    Document *mDocument;
 };
 
 
@@ -77,6 +86,11 @@ inline Document *EditableAsset::document() const
     return mDocument;
 }
 
+inline void EditableAsset::setDocument(Document *document)
+{
+    mDocument = document;
+}
+
 } // namespace Tiled
 
 Q_DECLARE_METATYPE(Tiled::EditableAsset*)
