@@ -21,6 +21,7 @@
 #include "editablelayer.h"
 
 #include "changelayer.h"
+#include "editablegrouplayer.h"
 #include "editablemanager.h"
 #include "editablemap.h"
 #include "scriptmanager.h"
@@ -49,6 +50,12 @@ EditableMap *EditableLayer::map() const
     return asset()->isMap() ? static_cast<EditableMap*>(asset()) : nullptr;
 }
 
+EditableGroupLayer *EditableLayer::parentLayer() const
+{
+    GroupLayer *parent = layer()->parentLayer();
+    return static_cast<EditableGroupLayer*>(EditableManager::instance().editableLayer(map(), parent));
+}
+
 bool EditableLayer::isSelected() const
 {
     if (auto document = mapDocument())
@@ -109,7 +116,7 @@ Layer *EditableLayer::release()
 void EditableLayer::setName(const QString &name)
 {
     if (auto doc = document())
-        asset()->push(new SetLayerName(doc, layer(), name));
+        asset()->push(new SetLayerName(doc, { layer() }, name));
     else if (!checkReadOnly())
         layer()->setName(name);
 }
@@ -117,15 +124,23 @@ void EditableLayer::setName(const QString &name)
 void EditableLayer::setOpacity(qreal opacity)
 {
     if (auto doc = document())
-        asset()->push(new SetLayerOpacity(doc, layer(), opacity));
+        asset()->push(new SetLayerOpacity(doc, { layer() }, opacity));
     else if (!checkReadOnly())
         layer()->setOpacity(opacity);
 }
 
+void EditableLayer::setTintColor(const QColor &color)
+{
+    if (auto doc = document())
+        asset()->push(new SetLayerTintColor(doc, { layer() }, color));
+    else if (!checkReadOnly())
+        layer()->setTintColor(color);
+}
+
 void EditableLayer::setVisible(bool visible)
 {
     if (auto doc = document())
-        asset()->push(new SetLayerVisible(doc, layer(), visible));
+        asset()->push(new SetLayerVisible(doc, { layer() }, visible));
     else if (!checkReadOnly())
         layer()->setVisible(visible);
 }
@@ -133,7 +148,7 @@ void EditableLayer::setVisible(bool visible)
 void EditableLayer::setLocked(bool locked)
 {
     if (auto doc = document())
-        asset()->push(new SetLayerLocked(doc, layer(), locked));
+        asset()->push(new SetLayerLocked(doc, { layer() }, locked));
     else if (!checkReadOnly())
         layer()->setLocked(locked);
 }
@@ -141,11 +156,19 @@ void EditableLayer::setLocked(bool locked)
 void EditableLayer::setOffset(QPointF offset)
 {
     if (auto doc = document())
-        asset()->push(new SetLayerOffset(doc, layer(), offset));
+        asset()->push(new SetLayerOffset(doc, { layer() }, offset));
     else if (!checkReadOnly())
         layer()->setOffset(offset);
 }
 
+void EditableLayer::setParallaxFactor(QPointF factor)
+{
+    if (auto doc = document())
+        asset()->push(new SetLayerParallaxFactor(doc, { layer() }, factor));
+    else if (!checkReadOnly())
+        layer()->setParallaxFactor(factor);
+}
+
 void EditableLayer::setSelected(bool selected)
 {
     auto document = mapDocument();
@@ -174,3 +197,5 @@ MapDocument *EditableLayer::mapDocument() const
 }
 
 } // namespace Tiled
+
+#include "moc_editablelayer.cpp"
