@@ -35,18 +35,18 @@
 
 namespace Tiled {
 
-EditableTileset::EditableTileset(const QString &name,
-                                 QObject *parent)
+EditableTileset::EditableTileset(const QString &name, QObject *parent)
     : EditableAsset(nullptr, nullptr, parent)
     , mTileset(Tileset::create(name, 0, 0))
 {
     setObject(mTileset.data());
+    EditableManager::instance().mEditableTilesets.insert(tileset(), this);
 }
 
 EditableTileset::EditableTileset(const Tileset *tileset, QObject *parent)
     : EditableAsset(nullptr, const_cast<Tileset*>(tileset), parent)
     , mReadOnly(true)
-    , mTileset(tileset->sharedPointer())    // keep alive
+    , mTileset(const_cast<Tileset*>(tileset)->sharedFromThis())    // keep alive
 {
 }
 
@@ -60,6 +60,8 @@ EditableTileset::EditableTileset(TilesetDocument *tilesetDocument,
     connect(tilesetDocument, &TilesetDocument::tileObjectGroupChanged, this, &EditableTileset::tileObjectGroupChanged);
     connect(tilesetDocument->wangSetModel(), &TilesetWangSetModel::wangSetAdded, this, &EditableTileset::wangSetAdded);
     connect(tilesetDocument->wangSetModel(), &TilesetWangSetModel::wangSetRemoved, this, &EditableTileset::wangSetRemoved);
+
+    EditableManager::instance().mEditableTilesets.insert(tileset(), this);
 }
 
 EditableTileset::~EditableTileset()
@@ -173,18 +175,17 @@ void EditableTileset::removeTiles(const QList<QObject *> &tiles)
 
 EditableWangSet *EditableTileset::addWangSet(const QString &name, int type)
 {
+    if (checkReadOnly())
+        return nullptr;
+
     auto wangSet = std::make_unique<WangSet>(tileset(), name, static_cast<WangSet::Type>(type));
-    EditableWangSet *editable = nullptr;
 
-    if (auto doc = tilesetDocument()) {
+    if (auto doc = tilesetDocument())
         push(new AddWangSet(doc, wangSet.release()));
-        editable = EditableManager::instance().editableWangSet(this, tileset()->wangSets().last());
-    } else if (!checkReadOnly()) {
+    else
         tileset()->addWangSet(std::move(wangSet));
-        editable = EditableManager::instance().editableWangSet(this, tileset()->wangSets().last());
-    }
 
-    return editable;
+    return EditableManager::instance().editableWangSet(this, tileset()->wangSets().last());
 }
 
 void EditableTileset::removeWangSet(EditableWangSet *editableWangSet)
@@ -208,6 +209,11 @@ TilesetDocument *EditableTileset::tilesetDocument() const
     return static_cast<TilesetDocument*>(document());
 }
 
+QSharedPointer<Document> EditableTileset::createDocument()
+{
+    return TilesetDocumentPtr::create(mTileset);
+}
+
 void EditableTileset::setName(const QString &name)
 {
     if (auto doc = tilesetDocument())
@@ -335,7 +341,7 @@ void EditableTileset::detachTiles(const QList<Tile *> &tiles)
 {
     const auto &editableManager = EditableManager::instance();
     for (Tile *tile : tiles) {
-        if (auto editable = editableManager.find(tile)) {
+        if (EditableTile *editable = editableManager.find(tile)) {
             Q_ASSERT(editable->tileset() == this);
             editable->detach();
         }
