@@ -21,6 +21,7 @@
 #include "editpolygontool.h"
 
 #include "addremovemapobject.h"
+#include "changemapobject.h"
 #include "changepolygon.h"
 #include "createpolygonobjecttool.h"
 #include "geometry.h"
@@ -83,6 +84,9 @@ void EditPolygonTool::activate(MapScene *scene)
     // selection, and by only updating the handles of the objects that changed.
     connect(mapDocument(), &MapDocument::selectedObjectsChanged,
             this, &EditPolygonTool::updateHandles);
+
+    connect(scene, &MapScene::parallaxParametersChanged,
+            this, &EditPolygonTool::updateHandles);
 }
 
 void EditPolygonTool::deactivate(MapScene *scene)
@@ -90,6 +94,9 @@ void EditPolygonTool::deactivate(MapScene *scene)
     disconnect(mapDocument(), &MapDocument::selectedObjectsChanged,
                this, &EditPolygonTool::updateHandles);
 
+    disconnect(scene, &MapScene::parallaxParametersChanged,
+               this, &EditPolygonTool::updateHandles);
+
     abortCurrentAction();
 
     // Delete all handles
@@ -202,23 +209,7 @@ void EditPolygonTool::mousePressed(QGraphicsSceneMouseEvent *event)
         mMousePressed = true;
         mStart = event->scenePos();
         mScreenStart = event->screenPos();
-
-        const QList<QGraphicsItem *> items = mapScene()->items(mStart,
-                                                               Qt::IntersectsItemShape,
-                                                               Qt::DescendingOrder,
-                                                               viewTransform(event));
-
-        mClickedObject = nullptr;
-        for (QGraphicsItem *item : items) {
-            if (!item->isEnabled())
-                continue;
-            if (auto mapObjectItem = qgraphicsitem_cast<MapObjectItem*>(item)) {
-                if (mapObjectItem->mapObject()->objectGroup()->isUnlocked()) {
-                    mClickedObject = mapObjectItem->mapObject();
-                    break;
-                }
-            }
-        }
+        mClickedObject = topMostMapObjectAt(mStart);
         break;
     }
     case Qt::RightButton: {
@@ -428,7 +419,7 @@ void EditPolygonTool::updateHandles()
 
         QPointF objectScreenPos = renderer->pixelToScreenCoords(object->position());
         QTransform rotate = rotateAt(objectScreenPos, object->rotation());
-        QPointF totalOffset = object->objectGroup()->totalOffset();
+        QPointF totalOffset = mapScene()->absolutePositionForLayer(*object->objectGroup());
 
         // Update the position of all handles
         for (int i = 0; i < pointHandles.size(); ++i) {
@@ -481,7 +472,10 @@ void EditPolygonTool::updateSelection(QGraphicsSceneMouseEvent *event)
                 selectedObjects.append(mapObjectItem->mapObject());
         }
 
-        mapDocument()->setSelectedObjects(selectedObjects);
+        filterMapObjects(selectedObjects);
+
+        if (!selectedObjects.isEmpty())
+            mapDocument()->setSelectedObjects(selectedObjects);
     } else {
         // Update the selected handles
         QSet<PointHandle*> selectedHandles;
@@ -558,7 +552,7 @@ void EditPolygonTool::updateMovingItems(const QPointF &pos,
         MapObject *object = handle->mapObject();
         QPointF objectScreenPos = renderer->pixelToScreenCoords(object->position());
         QTransform rotate = rotateAt(objectScreenPos, -object->rotation());
-        newScreenPos = rotate.map(newScreenPos - object->objectGroup()->totalOffset());
+        newScreenPos = rotate.map(newScreenPos - mapScene()->absolutePositionForLayer(*object->objectGroup()));
         QPointF newPixelPos = renderer->screenToPixelCoords(newScreenPos);
 
         // update the polygon
@@ -580,7 +574,7 @@ void EditPolygonTool::finishMoving(const QPointF &pos)
         return;
 
     QUndoStack *undoStack = mapDocument()->undoStack();
-    undoStack->beginMacro(tr("Move %n Point(s)", "", mSelectedHandles.size()));
+    undoStack->beginMacro(tr("Move %n Node(s)", "", mSelectedHandles.size()));
 
     // TODO: This isn't really optimal. Would be better to have a single undo
     // command that supports changing multiple map objects.
@@ -657,7 +651,7 @@ void EditPolygonTool::showHandleContextMenu(QPoint screenPos)
 
     bool canDeleteSegment = false;
     if (n == 2) {
-        const PointHandle *secondHandle = *(mSelectedHandles.constBegin() + 1);
+        const PointHandle *secondHandle = *(++mSelectedHandles.constBegin());
         const MapObject *secondMapObject = secondHandle->mapObject();
 
         int indexDifference = std::abs(firstHandle->pointIndex() - secondHandle->pointIndex());
@@ -707,7 +701,7 @@ QSet<PointHandle *> EditPolygonTool::clickedHandles() const
     return handles;
 }
 
-typedef QHash<MapObject*, RangeSet<int> > PointIndexesByObject;
+using PointIndexesByObject = QHash<MapObject *, RangeSet<int>>;
 static PointIndexesByObject
 groupIndexesByObject(const QSet<PointHandle*> &handles)
 {
@@ -773,7 +767,7 @@ void EditPolygonTool::changeEvent(const ChangeEvent &event)
 
     switch (event.type) {
     case ChangeEvent::LayerChanged:
-        if (static_cast<const LayerChangeEvent&>(event).properties & LayerChangeEvent::OffsetProperty)
+        if (static_cast<const LayerChangeEvent&>(event).properties & LayerChangeEvent::PositionProperties)
             updateHandles();
         break;
     case ChangeEvent::MapObjectsChanged: {
@@ -1005,8 +999,8 @@ void EditPolygonTool::deleteSegment()
     if (mSelectedHandles.size() != 2)
         return;
 
-    const auto &firstHandle = *mSelectedHandles.begin();
-    const auto &secondHandle = *(mSelectedHandles.begin() + 1);
+    PointHandle *firstHandle = *mSelectedHandles.constBegin();
+    PointHandle *secondHandle = *(++mSelectedHandles.constBegin());
 
     MapObject *mapObject = firstHandle->mapObject();
 
@@ -1043,7 +1037,8 @@ void EditPolygonTool::deleteSegment()
 
         mapDocument()->undoStack()->beginMacro(tr("Delete Segment"));
         mapDocument()->undoStack()->push(new ChangePolygon(mapDocument(), mapObject, newPolygon, polygon));
-        mapDocument()->undoStack()->push(new TogglePolygonPolyline(mapObject));
+        mapDocument()->undoStack()->push(new ChangeMapObject(mapDocument(), mapObject,
+                                                             MapObject::ShapeProperty, MapObject::Polyline));
         mapDocument()->undoStack()->endMacro();
     }
 }
@@ -1100,7 +1095,7 @@ void EditPolygonTool::updateHover(const QPointF &scenePos, QGraphicsSceneMouseEv
                     continue;
 
                 // Translate mouse position to local pixel coordinates...
-                const QPointF totalOffset = object->objectGroup()->totalOffset();
+                const QPointF totalOffset = mapScene()->absolutePositionForLayer(*object->objectGroup());
                 const QPointF objectScreenPos = renderer->pixelToScreenCoords(object->position());
                 const QTransform rotate = rotateAt(objectScreenPos, -object->rotation());
                 const QPointF rotatedMouseScenePos = rotate.map(scenePos - totalOffset);
@@ -1144,3 +1139,5 @@ void EditPolygonTool::updateHover(const QPointF &scenePos, QGraphicsSceneMouseEv
     mHoveredHandle = hoveredHandle;
     mHoveredSegment = hoveredSegment;
 }
+
+#include "moc_editpolygontool.cpp"
