@@ -46,7 +46,10 @@ FileFormat::Options ExportHelper::formatOptions() const
 SharedTileset ExportHelper::prepareExportTileset(const SharedTileset &tileset,
                                                  bool savingTileset) const
 {
-    if (!mOptions)
+    const bool hasExportSettings = !(tileset->exportFileName.isEmpty()
+                                     && tileset->exportFormat.isEmpty());
+
+    if (!mOptions && !hasExportSettings)
         return tileset;
 
     // When the tileset is embedded we're effectively always saving it
@@ -56,7 +59,8 @@ SharedTileset ExportHelper::prepareExportTileset(const SharedTileset &tileset,
         return tileset; // Leave external tileset alone
 
     if (savingTileset && !(mOptions & (Preferences::DetachTemplateInstances |
-                                       Preferences::ResolveObjectTypesAndProperties))) {
+                                       Preferences::ResolveObjectTypesAndProperties))
+            && !hasExportSettings) {
         // We're saving this tileset as-is, so leave it alone
         return tileset;
     }
@@ -66,6 +70,12 @@ SharedTileset ExportHelper::prepareExportTileset(const SharedTileset &tileset,
     SharedTileset exportTileset = tileset->clone();
     exportTileset->setOriginalTileset(tileset);
 
+    // We don't want to save the export options in the exported file
+    if (hasExportSettings) {
+        exportTileset->exportFileName.clear();
+        exportTileset->exportFormat.clear();
+    }
+
     if (mOptions.testFlag(Preferences::DetachTemplateInstances)) {
         for (Tile *tile : exportTileset->tiles()) {
             if (!tile->objectGroup())
@@ -92,25 +102,43 @@ SharedTileset ExportHelper::prepareExportTileset(const SharedTileset &tileset,
 
 const Map *ExportHelper::prepareExportMap(const Map *map, std::unique_ptr<Map> &exportMap) const
 {
+    const bool hasExportSettings = !(map->exportFileName.isEmpty()
+                                     && map->exportFormat.isEmpty());
+
     // If no export options are active, return the same map
-    if (!mOptions)
+    if (!(mOptions & ~Preferences::ExportMinimized) && !hasExportSettings)
         return map;
 
     // Make a copy to which export options are applied
     exportMap = map->clone();
 
-    if (mOptions.testFlag(Preferences::DetachTemplateInstances))
-        for (Layer *layer : exportMap->objectGroups())
-            for (MapObject *object : *static_cast<ObjectGroup*>(layer))
-                if (object->isTemplateInstance())
+    // We don't want to save the export options in the exported file
+    if (hasExportSettings) {
+        exportMap->exportFileName.clear();
+        exportMap->exportFormat.clear();
+    }
+
+    if (mOptions.testFlag(Preferences::DetachTemplateInstances)) {
+        for (Layer *layer : exportMap->objectGroups()) {
+            for (MapObject *object : *static_cast<ObjectGroup*>(layer)) {
+                if (object->isTemplateInstance()) {
+                    // In case of templated tile objects, the map may not yet
+                    // have a reference to the used tileset.
+                    if (Tile *tile = object->cell().tile())
+                        exportMap->addTileset(tile->tileset()->sharedFromThis());
+
                     object->detachFromTemplate();
+                }
+            }
+        }
+    }
 
     if (mOptions.testFlag(Preferences::ResolveObjectTypesAndProperties))
         for (Layer *layer : exportMap->objectGroups())
             for (MapObject *object : *static_cast<ObjectGroup*>(layer))
                 resolveTypeAndProperties(object);
 
-    auto tilesets = exportMap->tilesets();
+    const auto tilesets = exportMap->tilesets();    // needs a copy
     for (const SharedTileset &tileset : tilesets) {
         auto exportTileset = prepareExportTileset(tileset, false);
         if (exportTileset != tileset)
