@@ -36,7 +36,7 @@ FileEdit::FileEdit(QWidget *parent)
     , mErrorTextColor(Qt::red)
 {
     QHBoxLayout *layout = new QHBoxLayout(this);
-    layout->setMargin(0);
+    layout->setContentsMargins(0, 0, 0, 0);
     layout->setSpacing(0);
 
     mLineEdit = new QLineEdit(this);
@@ -149,3 +149,5 @@ void FileEdit::buttonClicked()
 }
 
 } // namespace Tiled
+
+#include "moc_fileedit.cpp"
