@@ -26,14 +26,13 @@
 #include <QUndoCommand>
 
 namespace Tiled {
-namespace Internal {
 
-class MapDocument;
+class Document;
 
 class FlipMapObjects : public QUndoCommand
 {
 public:
-    FlipMapObjects(MapDocument *mapDocument,
+    FlipMapObjects(Document *document,
                    const QList<MapObject *> &mapObjects,
                    FlipDirection flipDirection);
 
@@ -43,11 +42,15 @@ public:
 private:
     void flip();
 
-    MapDocument *mMapDocument;
+    Document *mDocument;
     const QList<MapObject *> mMapObjects;
     QPointF mObjectsCenter;
     FlipDirection mFlipDirection;
+
+    QVector<bool> mOldRotationStates;
+    QVector<bool> mNewRotationStates;
+    QVector<bool> mOldCellStates;
+    QVector<bool> mNewCellStates;
 };
 
-} // namespace Internal
 } // namespace Tiled
