@@ -23,7 +23,6 @@
 #include "commandlineparser.h"
 #include "exporthelper.h"
 #include "languagemanager.h"
-#include "logginginterface.h"
 #include "mainwindow.h"
 #include "mapdocument.h"
 #include "mapformat.h"
@@ -31,6 +30,7 @@
 #include "pluginmanager.h"
 #include "preferences.h"
 #include "scriptmanager.h"
+#include "sentryhelper.h"
 #include "stylehelper.h"
 #include "tiledapplication.h"
 #include "tileset.h"
@@ -47,18 +47,31 @@
 #include <memory>
 
 #ifdef Q_OS_WIN
+
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
-#if QT_VERSION >= 0x050700
+
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+#include <QtGui/private/qguiapplication_p.h>
+#include <QtGui/qpa/qplatformintegration.h>
+#elif QT_VERSION >= 0x050700
 #include <QtPlatformHeaders\QWindowsWindowFunctions>
-#endif // QT_VERSION >= 0x050700
+#endif
+
 #ifdef ERROR
 #undef ERROR
 #endif
+
 #endif // Q_OS_WIN
 
 using namespace Tiled;
 
+static QTextStream& stdOut()
+{
+    static QTextStream ts(stdout);
+    return ts;
+}
+
 namespace {
 
 class CommandLineHandler : public CommandLineParser
@@ -102,30 +115,6 @@ private:
     }
 };
 
-static const QtMessageHandler QT_DEFAULT_MESSAGE_HANDLER = qInstallMessageHandler(nullptr);
-
-static void messagesToConsole(QtMsgType type, const QMessageLogContext &context, const QString &msg)
-{
-    QString txt;
-    switch (type) {
-    case QtFatalMsg:
-        // program will quit so no point routing to the Console window
-        break;
-    case QtInfoMsg:
-    case QtDebugMsg:
-        INFO(qFormatLogMessage(type, context, msg));
-        break;
-    case QtWarningMsg:
-        WARNING(qFormatLogMessage(type, context, msg));
-        break;
-    case QtCriticalMsg:
-        ERROR(qFormatLogMessage(type, context, msg));
-        break;
-    }
-
-    (*QT_DEFAULT_MESSAGE_HANDLER)(type, context, msg);
-}
-
 static void initializePluginsAndExtensions()
 {
     PluginManager::instance()->loadPlugins();
@@ -248,8 +237,8 @@ void CommandLineHandler::showVersion()
 {
     if (!showedVersion) {
         showedVersion = true;
-        qInfo().noquote() << QApplication::applicationDisplayName()
-                          << QApplication::applicationVersion();
+        stdOut() << QApplication::applicationDisplayName() << " "
+                 << QApplication::applicationVersion() << Qt::endl;
         quit = true;
     }
 }
@@ -306,9 +295,9 @@ void CommandLineHandler::showExportFormats()
     }
     formats.sort(Qt::CaseSensitive);
 
-    qInfo().noquote() << tr("Map export formats:");
+    stdOut() << tr("Map export formats:") << Qt::endl;
     for (const QString &name : formats)
-        qInfo(" %s", qUtf8Printable(name));
+        stdOut() << " " << name << Qt::endl;
 
     formats.clear();
     const auto tilesetFormats = PluginManager::objects<TilesetFormat>();
@@ -318,9 +307,9 @@ void CommandLineHandler::showExportFormats()
     }
     formats.sort(Qt::CaseSensitive);
 
-    qInfo().noquote() << tr("Tileset export formats:");
+    stdOut() << tr("Tileset export formats:") << Qt::endl;
     for (const QString &name : formats)
-        qInfo(" %s", qUtf8Printable(name));
+        stdOut() << " " << name << Qt::endl;
 
     quit = true;
 }
@@ -333,7 +322,7 @@ void CommandLineHandler::startNewInstance()
 
 int main(int argc, char *argv[])
 {
-#if defined(Q_OS_WIN) && (!defined(Q_CC_MINGW) || __MINGW32_MAJOR_VERSION >= 5)
+#if defined(Q_OS_WIN) && (!defined(Q_CC_MINGW) || __GNUC__ >= 5)
     // Make console output work on Windows, if running in a console.
     if (AttachConsole(ATTACH_PARENT_PROCESS)) {
         FILE *dummy = nullptr;
@@ -342,15 +331,27 @@ int main(int argc, char *argv[])
     }
 #endif
 
-    qInstallMessageHandler(messagesToConsole);
+#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
+    QGuiApplication::setHighDpiScaleFactorRoundingPolicy(Qt::HighDpiScaleFactorRoundingPolicy::RoundPreferFloor);
 
+    // High-DPI scaling is always enabled in Qt 6
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
+#endif
+#endif
+
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    // Enable support for highres images (added in Qt 5.1, but off by default, always enabled in Qt 6)
+    QCoreApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
+
+    // Fallback session management was removed from Qt 6
     QGuiApplication::setFallbackSessionManagementEnabled(false);
 
-    // Enable support for highres images (added in Qt 5.1, but off by default)
-    QCoreApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
+    // Window context help buttons are disabled by default in Qt 6
 #if QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)
     QCoreApplication::setAttribute(Qt::AA_DisableWindowContextHelpButton);
 #endif
+#endif
 
 #ifdef Q_OS_MAC
     QCoreApplication::setAttribute(Qt::AA_DontShowIconsInMenus);
@@ -358,6 +359,10 @@ int main(int argc, char *argv[])
 
     TiledApplication a(argc, argv);
 
+#ifdef TILED_SENTRY
+    Sentry sentry;
+#endif
+
     initializeMetatypes();
 
     // Add the built-in file formats
@@ -402,9 +407,11 @@ int main(int argc, char *argv[])
         }
 
         // Load the source file
-        const std::unique_ptr<Map> sourceMap(readMap(sourceFile, nullptr));
+        const std::unique_ptr<Map> sourceMap(readMap(sourceFile, &errorMsg));
         if (!sourceMap) {
             qWarning().noquote() << QCoreApplication::translate("Command line", "Failed to load source map.");
+            if (!errorMsg.isEmpty())
+                qWarning().noquote() << errorMsg;
             return 1;
         }
 
@@ -446,9 +453,11 @@ int main(int argc, char *argv[])
         }
 
         // Load the source file
-        SharedTileset sourceTileset(readTileset(sourceFile, nullptr));
+        SharedTileset sourceTileset(readTileset(sourceFile, &errorMsg));
         if (!sourceTileset) {
             qWarning().noquote() << QCoreApplication::translate("Command line", "Failed to load source tileset.");
+            if (!errorMsg.isEmpty())
+                qWarning().noquote() << errorMsg;
             return 1;
         }
 
@@ -498,8 +507,14 @@ int main(int argc, char *argv[])
     w.show();
 
     a.setActivationWindow(&w);
-#if defined(Q_OS_WIN) && QT_VERSION >= 0x050700
+#ifdef Q_OS_WIN
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    using QWindowsApplication = QNativeInterface::Private::QWindowsApplication;
+    if (auto nativeWindowsApp = dynamic_cast<QWindowsApplication *>(QGuiApplicationPrivate::platformIntegration()))
+        nativeWindowsApp->setWindowActivationBehavior(QWindowsApplication::AlwaysActivateWindow);
+#elif QT_VERSION >= 0x050700
     QWindowsWindowFunctions::setWindowActivationBehavior(QWindowsWindowFunctions::AlwaysActivateWindow);
+#endif
 #endif
 
     QObject::connect(&a, &TiledApplication::fileOpenRequest,
