@@ -58,6 +58,7 @@ class MapEditor;
 class MapScene;
 class MapView;
 class ObjectTypesEditor;
+class PropertyTypesEditor;
 class ProjectDock;
 class ProjectModel;
 class TilesetDocument;
@@ -188,7 +189,7 @@ private:
     void autoMappingWarning(bool automatic);
 
     void onObjectTypesEditorClosed();
-
+    void onPropertyTypesEditorClosed();
     void ensureHasBorderInFullScreen();
 
     /**
@@ -236,6 +237,7 @@ private:
     ProjectDock *mProjectDock;
     IssuesDock *mIssuesDock;
     ObjectTypesEditor *mObjectTypesEditor;
+    PropertyTypesEditor *mPropertyTypesEditor;
     QPointer<LocatorWidget> mLocatorWidget;
     QPointer<QWidget> mPopupWidget;
     double mPopupWidgetShowProgress = 1.0;
@@ -248,7 +250,7 @@ private:
     QMenu *mViewsAndToolbarsMenu;
     QAction *mViewsAndToolbarsAction;
     QAction *mShowObjectTypesEditor;
-
+    QAction *mShowPropertyTypesEditor;
     QAction *mResetToDefaultLayout;
     QAction *mLockLayout;
 
