@@ -27,8 +27,7 @@
 namespace Tiled {
 
 class MapObject;
-
-namespace Internal {
+class Tile;
 
 class Handle;
 class MapDocument;
@@ -51,19 +50,21 @@ public:
      * @param parent the item of the object group this object belongs to
      */
     MapObjectItem(MapObject *object, MapDocument *mapDocument,
-                  ObjectGroupItem *parent = nullptr);
+                  QGraphicsItem *parent = nullptr);
 
     enum { Type = UserType + 1 };
     int type() const override { return Type; }
 
-    MapObject *mapObject() const
-    { return mObject; }
+    MapObject *mapObject() const;
 
     /**
      * Should be called when the map object this item refers to was changed.
      */
     void syncWithMapObject();
 
+    bool isHoverIndicator() const;
+    void setIsHoverIndicator(bool isHoverIndicator);
+
     // QGraphicsItem
     QRectF boundingRect() const override;
     QPainterPath shape() const override;
@@ -72,26 +73,15 @@ public:
                const QStyleOptionGraphicsItem *option,
                QWidget *widget = nullptr) override;
 
-    /**
-     * Resizes the associated map object. The \a bounds are given in pixel
-     * coordinates.
-     */
-    void resizeObject(const QRectF &bounds);
-
     /**
      * Sets a new polygon on the associated object.
      */
     void setPolygon(const QPolygonF &polygon);
 
-    /**
-     * A helper function to determine the color of a map object. The color is
-     * determined first of all by the object type, and otherwise by the group
-     * that the object is in. If still no color is defined, it defaults to
-     * gray.
-     */
-    static QColor objectColor(const MapObject *object);
-
 private:
+    void expandBoundsToCoverTileCollisionObjects(QRectF &bounds);
+    QTransform tileCollisionObjectsTransform(const Tile &tile) const;
+
     MapDocument *mapDocument() const { return mMapDocument; }
     QColor color() const { return mColor; }
 
@@ -100,12 +90,22 @@ private:
 
     /** Bounding rect cached, for adapting to geometry change correctly. */
     QRectF mBoundingRect;
-    QString mName;      // Copy of the name, so we know when it changes
-    QPolygonF mPolygon; // Copy of the polygon, for the same reason
+    QPolygonF mPolygon; // Copy of the polygon, so we know when it changes
     QColor mColor;      // Cached color of the object
+    bool mIsHoveredIndicator = false;
 };
 
-} // namespace Internal
+
+inline MapObject *MapObjectItem::mapObject() const
+{
+    return mObject;
+}
+
+inline bool MapObjectItem::isHoverIndicator() const
+{
+    return mIsHoveredIndicator;
+}
+
 } // namespace Tiled
 
-Q_DECLARE_METATYPE(Tiled::Internal::MapObjectItem*)
+Q_DECLARE_METATYPE(Tiled::MapObjectItem*)
