@@ -20,35 +20,13 @@
 
 #include "objectgroupitem.h"
 
-#include "map.h"
-#include "mapobjectitem.h"
-#include "objectgroup.h"
-
 using namespace Tiled;
-using namespace Tiled::Internal;
 
-ObjectGroupItem::ObjectGroupItem(ObjectGroup *objectGroup):
-    mObjectGroup(objectGroup)
+ObjectGroupItem::ObjectGroupItem(ObjectGroup *objectGroup, QGraphicsItem *parent)
+    : LayerItem(objectGroup, parent)
 {
     // Since we don't do any painting, we can spare us the call to paint()
     setFlag(QGraphicsItem::ItemHasNoContents);
-
-    setOpacity(objectGroup->opacity());
-    setPos(objectGroup->offset());
-}
-
-/*
- * Changes the object group represented by this item. Currently only expected
- * to be used by the CreateObjectTool.
- */
-void ObjectGroupItem::setObjectGroup(ObjectGroup *objectGroup)
-{
-    if (mObjectGroup == objectGroup)
-        return;
-
-    mObjectGroup = objectGroup;
-    setOpacity(mObjectGroup->opacity());
-    setPos(mObjectGroup->offset());
 }
 
 QRectF ObjectGroupItem::boundingRect() const
