@@ -22,15 +22,15 @@
 #include "ui_objecttypeseditor.h"
 
 #include "addpropertydialog.h"
+#include "custompropertieshelper.h"
 #include "object.h"
 #include "objecttypesmodel.h"
 #include "preferences.h"
+#include "properties.h"
 #include "session.h"
 #include "utils.h"
-#include "varianteditorfactory.h"
-#include "variantpropertymanager.h"
 
-#include <QtGroupPropertyManager>
+#include <QtVariantProperty>
 
 #include <QCloseEvent>
 #include <QColorDialog>
@@ -99,10 +99,11 @@ ObjectTypesEditor::ObjectTypesEditor(QWidget *parent)
     : QDialog(parent)
     , mUi(new Ui::ObjectTypesEditor)
     , mObjectTypesModel(new ObjectTypesModel(this))
-    , mVariantManager(new VariantPropertyManager(this))
-    , mGroupManager(new QtGroupPropertyManager(this))
 {
     mUi->setupUi(this);
+
+    mPropertiesHelper = new CustomPropertiesHelper(mUi->propertiesView, this);
+
     resize(Utils::dpiScaled(size()));
 
     mUi->objectTypesTable->setModel(mObjectTypesModel);
@@ -113,7 +114,6 @@ ObjectTypesEditor::ObjectTypesEditor(QWidget *parent)
     horizontalHeader->setSectionResizeMode(1, QHeaderView::Fixed);
     horizontalHeader->resizeSection(1, Utils::dpiScaled(50));
 
-    mUi->propertiesView->setFactoryForManager(mVariantManager, new VariantEditorFactory(this));
     mUi->propertiesView->setResizeMode(QtTreePropertyBrowser::ResizeToContents);
     mUi->propertiesView->setRootIsDecorated(false);
     mUi->propertiesView->setPropertiesWithoutValueMarked(true);
@@ -195,7 +195,7 @@ ObjectTypesEditor::ObjectTypesEditor(QWidget *parent)
     connect(mObjectTypesModel, &ObjectTypesModel::rowsRemoved,
             this, &ObjectTypesEditor::applyObjectTypes);
 
-    connect(mVariantManager, &QtVariantPropertyManager::valueChanged,
+    connect(mPropertiesHelper, &CustomPropertiesHelper::propertyValueChanged,
             this, &ObjectTypesEditor::propertyValueChanged);
 
     connect(mUi->propertiesView, &QtTreePropertyBrowser::currentItemChanged,
@@ -422,11 +422,9 @@ void ObjectTypesEditor::updateProperties()
 
     mAddPropertyAction->setEnabled(!selectedRows.isEmpty());
 
-    mProperties = aggregatedProperties;
-
     QScopedValueRollback<bool> updating(mUpdating, true);
-    mVariantManager->clear();
-    mNameToProperty.clear();
+
+    mPropertiesHelper->clear();
 
     QMapIterator<QString, AggregatedPropertyData> it(aggregatedProperties);
     while (it.hasNext()) {
@@ -435,8 +433,8 @@ void ObjectTypesEditor::updateProperties()
         const QString &name = it.key();
         const AggregatedPropertyData &data = it.value();
 
-        QtVariantProperty *property = createProperty(data.value().userType(), name);
-        property->setValue(data.value());
+        QtProperty *property = mPropertiesHelper->createProperty(name, data.value());
+        mUi->propertiesView->addProperty(property);
 
         bool everywhere = data.presenceCount() == selectedRows.size();
         bool consistent = everywhere && data.valueConsistent();
@@ -447,33 +445,13 @@ void ObjectTypesEditor::updateProperties()
     }
 }
 
-void ObjectTypesEditor::propertyValueChanged(QtProperty *property,
+void ObjectTypesEditor::propertyValueChanged(const QString &name,
                                              const QVariant &value)
 {
     if (mUpdating)
         return;
-    if (!mUi->propertiesView->topLevelItem(property))
-        return;
 
-    applyPropertyToSelectedTypes(property->propertyName(), value);
-}
-
-QtVariantProperty *ObjectTypesEditor::createProperty(int type,
-                                                     const QString &name)
-{
-    QtVariantProperty *property = mVariantManager->addProperty(type, name);
-    if (!property) {
-        // fall back to string property for unsupported property types
-        property = mVariantManager->addProperty(QMetaType::QString, name);
-    }
-
-    if (type == QMetaType::Bool)
-        property->setAttribute(QLatin1String("textVisible"), false);
-
-    mUi->propertiesView->addProperty(property);
-    mNameToProperty.insert(name, property);
-
-    return property;
+    applyPropertyToSelectedTypes(name, value);
 }
 
 void ObjectTypesEditor::openAddPropertyDialog()
@@ -495,7 +473,7 @@ void ObjectTypesEditor::addProperty(const QString &name, const QVariant &value)
 
 void ObjectTypesEditor::editProperty(const QString &name)
 {
-    QtVariantProperty *property = mNameToProperty.value(name);
+    QtVariantProperty *property = mPropertiesHelper->property(name);
     if (!property)
         return;
 
@@ -521,8 +499,7 @@ void ObjectTypesEditor::removeProperty()
         }
     }
 
-    mProperties.remove(name);
-    delete mNameToProperty.take(name);
+    mPropertiesHelper->deleteProperty(item->property());
 
     removePropertyFromSelectedTypes(name);
 }
