@@ -20,22 +20,18 @@
 
 #pragma once
 
-#include "properties.h"
-
 #include <QDialog>
+#include <QVariant>
 
 namespace Ui {
 class ObjectTypesEditor;
 }
 
 class QtBrowserItem;
-class QtGroupPropertyManager;
-class QtProperty;
-class QtVariantProperty;
-class QtVariantPropertyManager;
 
 namespace Tiled {
 
+class CustomPropertiesHelper;
 class ObjectTypesModel;
 
 class ObjectTypesEditor : public QDialog
@@ -67,7 +63,7 @@ private:
     void exportObjectTypes();
 
     void updateProperties();
-    void propertyValueChanged(QtProperty *property, const QVariant &val);
+    void propertyValueChanged(const QString &name, const QVariant &val);
 
     void openAddPropertyDialog();
     void addProperty(const QString &name, const QVariant &value = QVariant());
@@ -81,17 +77,11 @@ private:
 
     void retranslateUi();
 
-    QtVariantProperty *createProperty(int type,
-                                      const QString &name);
-
     Ui::ObjectTypesEditor *mUi;
     ObjectTypesModel *mObjectTypesModel;
 
-    QtVariantPropertyManager *mVariantManager;
-    QtGroupPropertyManager *mGroupManager;
-    QHash<QString, QtVariantProperty *> mNameToProperty;
+    CustomPropertiesHelper *mPropertiesHelper;
 
-    AggregatedProperties mProperties;
     bool mUpdating = false;
     bool mSettingPrefObjectTypes = false;
 
