@@ -25,16 +25,20 @@
 #include <QRegion>
 #include <QUndoCommand>
 
+#include <memory>
+#include <unordered_map>
+
 namespace Tiled {
 
 class TileLayer;
 
-namespace Internal {
-
 class MapDocument;
 
 /**
  * A command that paints one tile layer on top of another tile layer.
+ *
+ * Can merge with additional commands, even when they paint on different
+ * tile layers.
  */
 class PaintTileLayer : public QUndoCommand
 {
@@ -85,12 +89,18 @@ public:
     bool mergeWith(const QUndoCommand *other) override;
 
 private:
+    struct LayerData
+    {
+        void mergeWith(const LayerData &o);
+
+        std::unique_ptr<TileLayer> mSource;
+        std::unique_ptr<TileLayer> mErased;
+        int mX, mY;
+        QRegion mPaintedRegion;
+    };
+
     MapDocument *mMapDocument;
-    TileLayer *mTarget;
-    TileLayer *mSource;
-    TileLayer *mErased;
-    int mX, mY;
-    QRegion mPaintedRegion;
+    std::unordered_map<TileLayer*, LayerData> mLayerData;
     bool mMergeable;
 };
 
@@ -99,5 +109,4 @@ inline void PaintTileLayer::setMergeable(bool mergeable)
     mMergeable = mergeable;
 }
 
-} // namespace Internal
 } // namespace Tiled
