@@ -49,19 +49,25 @@ public:
 
 private:
     Preferences();
+    Preferences(const QString &fileName);
     ~Preferences() override;
 
+    void initialize();
+
 public:
     bool showGrid() const;
     bool showTileObjectOutlines() const;
     bool showTileAnimations() const;
     bool showTileCollisionShapes() const;
     bool showObjectReferences() const;
+    bool parallaxEnabled() const;
     bool snapToGrid() const;
     bool snapToFineGrid() const;
     bool snapToPixels() const;
     QColor gridColor() const;
+    QColor backgroundFadeColor() const;
     int gridFine() const;
+    QSize gridMajor() const;
     qreal objectLineWidth() const;
 
     bool highlightCurrentLayer() const;
@@ -129,19 +135,7 @@ public:
     void setUseOpenGL(bool useOpenGL);
 
     void setObjectTypes(const ObjectTypes &objectTypes);
-
-    enum FileType {
-        ExportedFile,
-        ExternalTileset,
-        ImageFile,
-        ObjectTemplateFile,
-        ObjectTypesFile,
-        ProjectFile,
-        WorldFile,
-    };
-
-    QString lastPath(FileType fileType) const;
-    void setLastPath(FileType fileType, const QString &path);
+    void setPropertyTypes(const SharedPropertyTypes &propertyTypes);
 
     QString objectTypesFile() const;
     void setObjectTypesFile(const QString &filePath);
@@ -153,14 +147,15 @@ public:
     bool isPatron() const;
     void setPatron(bool isPatron);
 
-    bool shouldShowDonationDialog() const;
-    QDate donationDialogTime() const;
-    void setDonationDialogReminder(const QDate &date);
+    bool shouldShowDonationReminder() const;
+    QDate donationReminderTime() const;
+    void setDonationReminder(const QDate &date);
 
     enum { MaxRecentFiles = 12 };
     void addRecentFile(const QString &fileName);
 
     QStringList recentProjects() const;
+    QString recentProjectPath() const;
     void addRecentProject(const QString &fileName);
 
     QString startupSession() const;
@@ -179,8 +174,9 @@ public:
     T get(const char *key, const T &defaultValue = T()) const
     { return value(QLatin1String(key), defaultValue).template value<T>(); }
 
-    static QString dataLocation();
-    static QString configLocation();
+    static QString homeLocation();
+    QString dataLocation() const;
+    QString configLocation() const;
 
     static QString startupProject();
     static void setStartupProject(const QString &filePath);
@@ -191,11 +187,16 @@ public slots:
     void setShowTileAnimations(bool enabled);
     void setShowTileCollisionShapes(bool enabled);
     void setShowObjectReferences(bool enabled);
+    void setParallaxEnabled(bool enabled);
     void setSnapToGrid(bool snapToGrid);
     void setSnapToFineGrid(bool snapToFineGrid);
     void setSnapToPixels(bool snapToPixels);
     void setGridColor(QColor gridColor);
+    void setBackgroundFadeColor(QColor backgroundFadeColor);
     void setGridFine(int gridFine);
+    void setGridMajor(QSize gridMajor);
+    void setGridMajorX(int gridMajorX);
+    void setGridMajorY(int gridMajorY);
     void setObjectLineWidth(qreal lineWidth);
     void setHighlightCurrentLayer(bool highlight);
     void setHighlightHoveredObject(bool highlight);
@@ -213,11 +214,14 @@ signals:
     void showTileAnimationsChanged(bool enabled);
     void showTileCollisionShapesChanged(bool enabled);
     void showObjectReferencesChanged(bool enabled);
+    void parallaxEnabledChanged(bool enabled);
     void snapToGridChanged(bool snapToGrid);
     void snapToFineGridChanged(bool snapToFineGrid);
     void snapToPixelsChanged(bool snapToPixels);
     void gridColorChanged(QColor gridColor);
+    void backgroundFadeColorChanged(QColor backgroundFadeColor);
     void gridFineChanged(int gridFine);
+    void gridMajorChanged(QSize gridMajor);
     void objectLineWidthChanged(qreal lineWidth);
     void highlightCurrentLayerChanged(bool highlight);
     void highlightHoveredObjectChanged(bool highlight);
@@ -235,6 +239,8 @@ signals:
 
     void objectTypesChanged();
 
+    void propertyTypesChanged();
+
     void isPatronChanged();
 
     void recentFilesChanged();
@@ -251,6 +257,7 @@ private:
     void objectTypesFileChangedOnDisk();
 
     FileSystemWatcher mWatcher;
+    bool mPortable = false;
 
     QString mObjectTypesFile;
     QDateTime mObjectTypesFileLastSaved;
