@@ -1,6 +1,6 @@
 /*
  * propertybrowser.h
- * Copyright 2013, Thorbjørn Lindeijer <thorbjorn@lindeijer.nl>
+ * Copyright 2013-2021, Thorbjørn Lindeijer <thorbjorn@lindeijer.nl>
  *
  * This file is part of Tiled.
  *
@@ -21,6 +21,7 @@
 #pragma once
 
 #include "changeevents.h"
+#include "custompropertieshelper.h"
 #include "map.h"
 #include "properties.h"
 
@@ -75,11 +76,10 @@ private:
     void documentChanged(const ChangeEvent &change);
     void mapChanged();
     void objectsChanged(const MapObjectsChangeEvent &mapObjectsChange);
-    void imageLayerChanged(ImageLayer *imageLayer);
     void tilesetChanged(Tileset *tileset);
     void tileChanged(Tile *tile);
     void tileTypeChanged(Tile *tile);
-    void wangSetChanged(Tileset *tileset, int index);
+    void wangSetChanged(WangSet *wangSet);
 
     void propertyAdded(Object *object, const QString &name);
     void propertyRemoved(Object *object, const QString &name);
@@ -92,6 +92,7 @@ private:
     void objectTypesChanged();
 
     void valueChanged(QtProperty *property, const QVariant &val);
+    void customPropertyValueChanged(const QString &name, const QVariant &value);
 
     void resetProperty(QtProperty *property);
 
@@ -113,6 +114,8 @@ private:
         OffsetXProperty,
         OffsetYProperty,
         ParallaxFactorProperty,
+        RepeatXProperty,
+        RepeatYProperty,
         ColorProperty,
         BackgroundColorProperty,
         TileWidthProperty,
@@ -123,6 +126,7 @@ private:
         HexSideLengthProperty,
         StaggerAxisProperty,
         StaggerIndexProperty,
+        ParallaxOriginProperty,
         RenderOrderProperty,
         LayerFormatProperty,
         ImageSourceProperty,
@@ -140,7 +144,6 @@ private:
         ColorCountProperty,
         WangColorProbabilityProperty,
         WangSetTypeProperty,
-        CustomProperty,
         InfiniteProperty,
         TemplateProperty,
         CompressionLevelProperty,
@@ -169,11 +172,10 @@ private:
     void applyMapObjectValue(PropertyId id, const QVariant &val);
     QUndoCommand *applyMapObjectValueTo(PropertyId id, const QVariant &val, MapObject *mapObject);
     void applyLayerValue(PropertyId id, const QVariant &val);
-    QUndoCommand *applyLayerValueTo(PropertyId id, const QVariant &val, Layer *layer);
-    QUndoCommand *applyTileLayerValueTo(PropertyId id, const QVariant &val, TileLayer *tileLayer);
-    QUndoCommand *applyObjectGroupValueTo(PropertyId id, const QVariant &val, ObjectGroup *objectGroup);
-    QUndoCommand *applyImageLayerValueTo(PropertyId id, const QVariant &val, ImageLayer *imageLayer);
-    QUndoCommand *applyGroupLayerValueTo(PropertyId id, const QVariant &val, GroupLayer *groupLayer);
+    QUndoCommand *applyTileLayerValueTo(PropertyId id, const QVariant &val, QList<TileLayer *> tileLayers);
+    QUndoCommand *applyObjectGroupValueTo(PropertyId id, const QVariant &val, QList<ObjectGroup *> objectGroups);
+    QUndoCommand *applyImageLayerValueTo(PropertyId id, const QVariant &val, QList<ImageLayer *> imageLayers);
+    QUndoCommand *applyGroupLayerValueTo(PropertyId id, const QVariant &val, QList<GroupLayer *> groupLayers);
     void applyTilesetValue(PropertyId id, const QVariant &val);
     void applyTileValue(PropertyId id, const QVariant &val);
     void applyWangSetValue(PropertyId id, const QVariant &val);
@@ -182,6 +184,8 @@ private:
     QtVariantProperty *createProperty(PropertyId id,
                                       int type,
                                       const QString &name);
+    QtVariantProperty *createCustomProperty(const QString &name,
+                                            const QVariant &value);
 
     using QtTreePropertyBrowser::addProperty;
     QtVariantProperty *addProperty(PropertyId id,
@@ -189,9 +193,9 @@ private:
                                    const QString &name,
                                    QtProperty *parent);
 
-    QtVariantProperty *createCustomProperty(const QString &name, const QVariant &value);
-    void deleteCustomProperty(QtVariantProperty *property);
+    QtVariantProperty *addCustomProperty(const QString &name, const QVariant &value);
     void setCustomPropertyValue(QtVariantProperty *property, const QVariant &value);
+    void recreateProperty(QtVariantProperty *property, const QVariant &value);
 
     void addProperties();
     void removeProperties();
@@ -199,8 +203,8 @@ private:
     void updateCustomProperties();
     void updateCustomPropertyColor(const QString &name);
 
-    QVariant toDisplayValue(const QVariant &value) const;
-    QVariant fromDisplayValue(const QVariant &value) const;
+    QVariant toDisplayValue(QVariant value) const;
+    QVariant fromDisplayValue(QtProperty *property, QVariant value) const;
 
     void retranslateUi();
 
@@ -217,7 +221,7 @@ private:
 
     QHash<QtProperty *, PropertyId> mPropertyToId;
     QHash<PropertyId, QtVariantProperty *> mIdToProperty;
-    QHash<QString, QtVariantProperty *> mNameToProperty;
+    CustomPropertiesHelper mCustomPropertiesHelper;
 
     Properties mCombinedProperties;
 
