@@ -22,10 +22,6 @@
 
 namespace Tiled {
 
-RegionValueType::RegionValueType()
-{
-}
-
 RegionValueType::RegionValueType(int x, int y, int w, int h)
     : mRegion(x, y, w, h)
 {
@@ -56,4 +52,20 @@ QString RegionValueType::toString() const
     }
 }
 
+QVector<QRect> RegionValueType::rects() const
+{
+#if QT_VERSION < 0x050800
+    return mRegion.rects();
+#elif QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
+    QVector<QRect> rects;
+    rects.reserve(static_cast<int>(mRegion.end() - mRegion.begin()));
+    for (const QRect &rect : mRegion)
+        rects.append(rect);
+#else
+    return QVector<QRect>(mRegion.begin(), mRegion.end());
+#endif
+}
+
 } // namespace Tiled
+
+#include "moc_regionvaluetype.cpp"
