@@ -20,44 +20,49 @@
 
 #include "rotatemapobject.h"
 
-#include "mapdocument.h"
+#include "changeevents.h"
+#include "document.h"
 #include "mapobject.h"
-#include "mapobjectmodel.h"
 
 #include <QCoreApplication>
 
 using namespace Tiled;
-using namespace Tiled::Internal;
 
-RotateMapObject::RotateMapObject(MapDocument *mapDocument,
+RotateMapObject::RotateMapObject(Document *document,
                                  MapObject *mapObject,
                                  qreal oldRotation)
-    : mMapDocument(mapDocument)
-    , mMapObject(mapObject)
-    , mOldRotation(oldRotation)
-    , mNewRotation(mapObject->rotation())
+    : RotateMapObject(document,
+                      mapObject,
+                      mapObject->rotation(),
+                      oldRotation)
 {
-    setText(QCoreApplication::translate("Undo Commands", "Rotate Object"));
 }
 
-RotateMapObject::RotateMapObject(MapDocument *mapDocument,
+RotateMapObject::RotateMapObject(Document *document,
                                  MapObject *mapObject,
                                  qreal newRotation,
                                  qreal oldRotation)
-    : mMapDocument(mapDocument)
+    : mDocument(document)
     , mMapObject(mapObject)
     , mOldRotation(oldRotation)
     , mNewRotation(newRotation)
+    , mOldChangeState(mapObject->propertyChanged(MapObject::RotationProperty))
 {
     setText(QCoreApplication::translate("Undo Commands", "Rotate Object"));
 }
 
 void RotateMapObject::undo()
 {
-    mMapDocument->mapObjectModel()->setObjectRotation(mMapObject, mOldRotation);
+    mMapObject->setRotation(mOldRotation);
+    mMapObject->setPropertyChanged(MapObject::RotationProperty, mOldChangeState);
+
+    emit mDocument->changed(MapObjectsChangeEvent(mMapObject, MapObject::RotationProperty));
 }
 
 void RotateMapObject::redo()
 {
-    mMapDocument->mapObjectModel()->setObjectRotation(mMapObject, mNewRotation);
+    mMapObject->setRotation(mNewRotation);
+    mMapObject->setPropertyChanged(MapObject::RotationProperty);
+
+    emit mDocument->changed(MapObjectsChangeEvent(mMapObject, MapObject::RotationProperty));
 }
