@@ -52,13 +52,10 @@ StampBrush::StampBrush(QObject *parent)
                        QKeySequence(Qt::Key_B),
                        nullptr,
                        parent)
-    , mBrushBehavior(Free)
-    , mIsRandom(false)
-    , mIsWangFill(false)
-    , mWangSet(nullptr)
-    , mRandomCacheValid(true)
     , mStampActions(new StampActions(this))
 {
+    setUsesSelectedTiles(true);
+
     connect(mStampActions->random(), &QAction::toggled, this, &StampBrush::randomChanged);
     connect(mStampActions->wangFill(), &QAction::toggled, this, &StampBrush::wangFillChanged);
 
@@ -78,6 +75,7 @@ StampBrush::~StampBrush()
 
 void StampBrush::deactivate(MapScene *scene)
 {
+    mBrushBehavior = Free;
     mCaptureStampHelper.reset();
     AbstractTileTool::deactivate(scene);
 }
@@ -137,7 +135,6 @@ void StampBrush::mousePressed(QGraphicsSceneMouseEvent *event)
                 break;
             case Free:
                 beginPaint();
-                mBrushBehavior = Paint;
                 break;
             case Capture:
                 break;
@@ -252,11 +249,6 @@ void StampBrush::mapDocumentChanged(MapDocument *oldDocument,
     }
 }
 
-static TileLayer *findTileLayerByName(const Map &map, const QString &name)
-{
-    return static_cast<TileLayer*>(map.findLayer(name, Layer::TileLayerType));
-}
-
 QList<Layer *> StampBrush::targetLayers() const
 {
     if (mIsRandom || mIsWangFill || mStamp.isEmpty())
@@ -279,7 +271,7 @@ void StampBrush::updateRandomList()
     mMissingTilesets.clear();
 
     for (const TileStampVariation &variation : mStamp.variations()) {
-        mapDocument()->unifyTilesets(variation.map, mMissingTilesets);
+        mapDocument()->unifyTilesets(*variation.map, mMissingTilesets);
 
         for (auto layer : variation.map->tileLayers())
             for (const Cell &cell : *static_cast<TileLayer*>(layer))
@@ -313,7 +305,7 @@ void StampBrush::setWangSet(WangSet *wangSet)
     if (!wangSet)
         return;
 
-    const SharedTileset &tileset = wangSet->tileset()->sharedPointer();
+    const SharedTileset &tileset = wangSet->tileset()->sharedFromThis();
 
     if (!mapDocument() || !mapDocument()->map()->tilesets().contains(tileset))
        mMissingTilesets.append(tileset);
@@ -368,7 +360,7 @@ void StampBrush::doPaint(int flags, QHash<TileLayer*, QRegion> *paintedRegions)
     if (!preview)
         return;
 
-    mapDocument()->paintTileLayers(preview.data(),
+    mapDocument()->paintTileLayers(*preview,
                                    (flags & Mergeable) == Mergeable,
                                    &mMissingTilesets,
                                    paintedRegions);
@@ -422,9 +414,11 @@ void StampBrush::drawPreviewLayer(const QVector<QPoint> &points)
         for (const QPoint &p : points)
             bounds |= QRect(p, p);
 
-        SharedMap preview = SharedMap::create(mapDocument()->map()->orientation(),
-                                              bounds.size(),
-                                              mapDocument()->map()->tileSize());
+        Map::Parameters mapParameters = mapDocument()->map()->parameters();
+        mapParameters.width = bounds.width();
+        mapParameters.height = bounds.height();
+        mapParameters.infinite = false;
+        SharedMap preview = SharedMap::create(mapParameters);
 
         std::unique_ptr<TileLayer> previewLayer {
             new TileLayer(QString(), bounds.topLeft(), bounds.size())
@@ -453,43 +447,36 @@ void StampBrush::drawPreviewLayer(const QVector<QPoint> &points)
             paintedRegion += QRect(p, p);
 
         const QRect bounds = paintedRegion.boundingRect();
-        SharedMap preview = SharedMap::create(mapDocument()->map()->orientation(),
-                                              bounds.size(),
-                                              mapDocument()->map()->tileSize());
+
+        Map::Parameters mapParameters = mapDocument()->map()->parameters();
+        mapParameters.width = bounds.width();
+        mapParameters.height = bounds.height();
+        mapParameters.infinite = false;
+        SharedMap preview = SharedMap::create(mapParameters);
 
         std::unique_ptr<TileLayer> previewLayer {
             new TileLayer(QString(), bounds.topLeft(), bounds.size())
         };
 
-        WangFiller wangFiller(mWangSet,
-                              dynamic_cast<StaggeredRenderer *>(mapDocument()->renderer()),
-                              mapDocument()->map()->staggerAxis());
+        WangFiller wangFiller(*mWangSet, mapDocument()->renderer());
 
-        for (const QPoint &p : points) {
-            Cell cell = wangFiller.findFittingCell(*tileLayer,
-                                                   *previewLayer,
-                                                   paintedRegion,
-                                                   p);
-
-            previewLayer->setCell(p.x() - bounds.left(),
-                                  p.y() - bounds.top(),
-                                  cell);
-        }
+        wangFiller.fillRegion(*previewLayer, *tileLayer, paintedRegion);
 
         preview->addLayer(std::move(previewLayer));
-        preview->addTileset(mWangSet->tileset()->sharedPointer());
+        preview->addTileset(mWangSet->tileset()->sharedFromThis());
         mPreviewMap = preview;
     } else {
         QRegion paintedRegion;
         QVector<PaintOperation> operations;
         QHash<const Map *, QRegion> regionCache;
         QHash<const Map *, Map *> shiftedCopies;
+        const auto randomVariations = mStamp.randomVariations();
 
         mMissingTilesets.clear();
 
         for (const QPoint &p : points) {
-            Map *map = mStamp.randomVariation().map;
-            mapDocument()->unifyTilesets(map, mMissingTilesets);
+            Map *map = randomVariations.pick();
+            mapDocument()->unifyTilesets(*map, mMissingTilesets);
 
             Map::StaggerAxis mapStaggerAxis = mapDocument()->map()->staggerAxis();
 
@@ -555,18 +542,34 @@ void StampBrush::drawPreviewLayer(const QVector<QPoint> &points)
         }
 
         const QRect bounds = paintedRegion.boundingRect();
-        SharedMap preview = SharedMap::create(mapDocument()->map()->orientation(),
-                                              bounds.size(),
-                                              mapDocument()->map()->tileSize());
+
+        Map::Parameters mapParameters = mapDocument()->map()->parameters();
+        mapParameters.width = bounds.width();
+        mapParameters.height = bounds.height();
+        mapParameters.infinite = false;
+        SharedMap preview = SharedMap::create(mapParameters);
+
+        QHash<QString, QList<TileLayer*>> targetLayersByName;
 
         for (const PaintOperation &op : operations) {
+            QHash<QString, int> targetLayersIndices;
+
             LayerIterator layerIterator(op.stamp, Layer::TileLayerType);
             while (auto tileLayer = static_cast<TileLayer*>(layerIterator.next())) {
-                TileLayer *target = findTileLayerByName(*preview, tileLayer->name());
-                if (!target) {
+                auto &targetLayerIndex = targetLayersIndices[tileLayer->name()];
+                auto &targetLayers = targetLayersByName[tileLayer->name()];
+                TileLayer *target = nullptr;
+
+                if (targetLayerIndex < targetLayers.size()) {
+                    target = targetLayers[targetLayerIndex];
+                } else {
                     target = new TileLayer(tileLayer->name(), bounds.topLeft(), bounds.size());
+                    targetLayers.append(target);
                     preview->addLayer(target);
                 }
+
+                ++targetLayerIndex;
+
                 target->merge(op.pos - bounds.topLeft() + tileLayer->position(), tileLayer);
             }
         }
@@ -664,3 +667,5 @@ void StampBrush::invalidateRandomCache()
 {
     mRandomCacheValid = false;
 }
+
+#include "moc_stampbrush.cpp"
