@@ -24,6 +24,8 @@
 #include "session.h"
 #include "utils.h"
 
+#include <QFontDatabase>
+
 namespace Tiled {
 
 namespace session {
@@ -40,9 +42,9 @@ TextEditorDialog::TextEditorDialog(QWidget *parent)
     setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
 #endif
 
-    connect(mUi->monospaceFont, &QAbstractButton::toggled, [this] (bool checked) {
-        mUi->plainTextEdit->setStyleSheet(checked ? QStringLiteral("font-family: monospace;")
-                                                  : QString());
+    connect(mUi->monospaceFont, &QAbstractButton::toggled, this, [this] (bool checked) {
+        mUi->plainTextEdit->setFont(checked ? QFontDatabase::systemFont(QFontDatabase::FixedFont)
+                                            : QGuiApplication::font());
         session::monospace.set(checked);
     });
 
