@@ -53,6 +53,7 @@
 #include "zoomable.h"
 
 #include <QActionGroup>
+#include <QBitmap>
 #include <QCloseEvent>
 #include <QComboBox>
 #include <QCoreApplication>
@@ -95,6 +96,14 @@ TileCollisionDock::TileCollisionDock(QWidget *parent)
     CreateObjectTool *polygonObjectsTool = new CreatePolygonObjectTool(this);
     CreateObjectTool *templatesTool = new CreateTemplateTool(this);
 
+    // Autodetection of tile extents
+    QIcon autoDetectMaskIcon(QLatin1String("://images/24/detect-bounding-box.png"));
+    autoDetectMaskIcon.addFile(QLatin1String("://images/48/detect-bounding-box.png"));
+    mActionAutoDetectMask = new QAction(this);
+    mActionAutoDetectMask->setEnabled(false);
+    mActionAutoDetectMask->setIcon(autoDetectMaskIcon);
+    connect(mActionAutoDetectMask, &QAction::triggered, this, &TileCollisionDock::autoDetectMask);
+
     QToolBar *toolsToolBar = new QToolBar(this);
     toolsToolBar->setObjectName(QLatin1String("TileCollisionDockToolBar"));
     toolsToolBar->setMovable(false);
@@ -109,6 +118,8 @@ TileCollisionDock::TileCollisionDock(QWidget *parent)
     toolsToolBar->addAction(mToolManager->registerTool(ellipseObjectsTool));
     toolsToolBar->addAction(mToolManager->registerTool(polygonObjectsTool));
     toolsToolBar->addAction(mToolManager->registerTool(templatesTool));
+    toolsToolBar->addSeparator();
+    toolsToolBar->addAction(mActionAutoDetectMask);
 
     mActionDuplicateObjects = new QAction(this);
     mActionDuplicateObjects->setIcon(QIcon(QLatin1String(":/images/16/stock-duplicate-16.png")));
@@ -140,7 +151,7 @@ TileCollisionDock::TileCollisionDock(QWidget *parent)
     mObjectsWidget->setVisible(false);
     auto objectsVertical = new QVBoxLayout(mObjectsWidget);
     objectsVertical->setSpacing(0);
-    objectsVertical->setMargin(0);
+    objectsVertical->setContentsMargins(0, 0, 0, 0);
     objectsVertical->addWidget(mObjectsView);
     objectsVertical->addWidget(objectsToolBar);
 
@@ -203,7 +214,7 @@ TileCollisionDock::TileCollisionDock(QWidget *parent)
     auto widget = new QWidget(this);
     auto vertical = new QVBoxLayout(widget);
     vertical->setSpacing(0);
-    vertical->setMargin(0);
+    vertical->setContentsMargins(0, 0, 0, 0);
     vertical->addLayout(horizontal);
     vertical->addWidget(mObjectsViewSplitter);
 
@@ -219,6 +230,7 @@ TileCollisionDock::TileCollisionDock(QWidget *parent)
     retranslateUi();
     selectedObjectsChanged();   // disables actions
 
+    ActionManager::registerAction(mActionAutoDetectMask, "AutoDetectMask");
     ActionManager::registerAction(mActionDuplicateObjects, "DuplicateObjects");
     ActionManager::registerAction(mActionRemoveObjects, "RemoveObjects");
     ActionManager::registerAction(mActionMoveUp, "MoveObjectsUp");
@@ -232,6 +244,29 @@ TileCollisionDock::~TileCollisionDock()
     setTile(nullptr);
 }
 
+/**
+ * Automatically detect the extents of the tile and append a simple
+ * rectangular collision mask.
+ */
+void TileCollisionDock::autoDetectMask()
+{
+    if (!mDummyMapDocument)
+        return;
+
+    const QPixmap &pixmap = mTile->image();
+    const QRect content = pixmap.hasAlphaChannel() ? QRegion(pixmap.mask()).boundingRect()
+                                                   : pixmap.rect();
+
+    // Create the rectangular collision shape
+    MapObject *newObject = new MapObject(QString(), QString(),
+                                         content.topLeft(),
+                                         content.size());
+
+    ObjectGroup *objectGroup = static_cast<ObjectGroup*>(mDummyMapDocument->map()->layerAt(1));
+    mDummyMapDocument->undoStack()->push(new AddMapObjects(mDummyMapDocument.data(), objectGroup, newObject));
+    mDummyMapDocument->setSelectedObjects({ newObject });
+}
+
 void TileCollisionDock::saveState()
 {
     preferences::objectsViewVisibility = QVariant::fromValue(mObjectsViewVisibility).toString();
@@ -258,6 +293,12 @@ void TileCollisionDock::setTilesetDocument(TilesetDocument *tilesetDocument)
                 this, &TileCollisionDock::tileObjectGroupChanged);
         connect(mTilesetDocument, &TilesetDocument::tilesetTileOffsetChanged,
                 this, &TileCollisionDock::tilesetTileOffsetChanged);
+        connect(mTilesetDocument, &TilesetDocument::tilesetChanged,
+                this, &TileCollisionDock::tilesetChanged);
+
+        mMapScene->setOverrideBackgroundColor(mTilesetDocument->tileset()->backgroundColor());
+    } else {
+        mMapScene->setOverrideBackgroundColor(QColor());
     }
 }
 
@@ -351,17 +392,23 @@ void TileCollisionDock::setTile(Tile *tile)
     auto previousDocument = mDummyMapDocument;
 
     mMapView->setEnabled(tile);
+    mActionAutoDetectMask->setEnabled(tile);
 
     if (tile) {
-        Map::Orientation orientation = Map::Orthogonal;
-        QSize tileSize = tile->size();
+        Map::Parameters mapParameters;
+        mapParameters.width = 1;
+        mapParameters.height = 1;
 
         if (tile->tileset()->orientation() == Tileset::Isometric) {
-            orientation = Map::Isometric;
-            tileSize = tile->tileset()->gridSize();
+            mapParameters.orientation = Map::Isometric;
+            mapParameters.tileWidth = tile->tileset()->gridSize().width();
+            mapParameters.tileHeight = tile->tileset()->gridSize().height();
+        } else {
+            mapParameters.tileWidth = tile->width();
+            mapParameters.tileHeight = tile->height();
         }
 
-        std::unique_ptr<Map> map { new Map(orientation, 1, 1, tileSize.width(), tileSize.height()) };
+        auto map = std::make_unique<Map>(mapParameters);
         map->addTileset(tile->sharedTileset());
 
         TileLayer *tileLayer = new TileLayer(QString(), 0, 0, 1, 1);
@@ -380,7 +427,6 @@ void TileCollisionDock::setTile(Tile *tile)
         map->addLayer(objectGroup);
 
         mDummyMapDocument = MapDocumentPtr::create(std::move(map));
-        mDummyMapDocument->setAllowHidingObjects(false);
         mDummyMapDocument->setAllowTileObjects(false);
         mDummyMapDocument->switchCurrentLayer(objectGroup);
 
@@ -404,7 +450,6 @@ void TileCollisionDock::setTile(Tile *tile)
         mToolManager->setMapDocument(nullptr);
     }
 
-
     emit dummyMapDocumentChanged(mDummyMapDocument.data());
 
     setHasSelectedObjects(false);
@@ -503,6 +548,11 @@ void TileCollisionDock::tilesetTileOffsetChanged(Tileset *tileset)
     emit mDummyMapDocument->changed(LayerChangeEvent(tileLayer, LayerChangeEvent::OffsetProperty));
 }
 
+void TileCollisionDock::tilesetChanged(Tileset *tileset)
+{
+    mMapScene->setOverrideBackgroundColor(tileset->backgroundColor());
+}
+
 void TileCollisionDock::cut()
 {
     if (!mTile)
@@ -672,6 +722,8 @@ void TileCollisionDock::retranslateUi()
 {
     setWindowTitle(QCoreApplication::translate("Tiled::MainWindow", "Tile Collision Editor"));
 
+    mActionAutoDetectMask->setText(tr("Detect Bounding Box"));
+
     mActionDuplicateObjects->setText(tr("Duplicate Objects"));
     mActionRemoveObjects->setText(tr("Remove Objects"));
     mActionMoveUp->setText(tr("Move Objects Up"));
@@ -680,3 +732,5 @@ void TileCollisionDock::retranslateUi()
 }
 
 } // namespace Tiled
+
+#include "moc_tilecollisiondock.cpp"
