@@ -106,7 +106,7 @@ TileStampsDock::TileStampsDock(TileStampManager *stampManager, QWidget *parent)
 
     QWidget *widget = new QWidget(this);
     QVBoxLayout *layout = new QVBoxLayout(widget);
-    layout->setMargin(0);
+    layout->setContentsMargins(0, 0, 0, 0);
 
     QToolBar *buttonContainer = new QToolBar;
     buttonContainer->setFloatable(false);
@@ -339,3 +339,5 @@ bool TileStampView::event(QEvent *event)
 }
 
 } // namespace Tiled
+
+#include "moc_tilestampsdock.cpp"
