@@ -23,11 +23,14 @@
 #include "rangeset.h"
 
 #include <QIcon>
+#include <QJsonParseError>
 #include <QSettings>
 #include <QString>
 
 #include <memory>
 
+#include "qtcompat_p.h"
+
 class QAction;
 class QKeyEvent;
 class QMenu;
@@ -74,6 +77,8 @@ void setThemeIcon(T *t, const char *name)
     setThemeIcon(t, QLatin1String(name));
 }
 
+QIcon colorIcon(const QColor &color, QSize size);
+
 void restoreGeometry(QWidget *widget);
 void saveGeometry(QWidget *widget);
 
@@ -97,5 +102,9 @@ void addOpenWithSystemEditorAction(QMenu &menu, const QString &fileName);
 QSettings::Format jsonSettingsFormat();
 std::unique_ptr<QSettings> jsonSettings(const QString &fileName);
 
+namespace Error {
+QString jsonParseError(QJsonParseError error);
+} // namespace Error
+
 } // namespace Utils
 } // namespace Tiled
