@@ -98,7 +98,7 @@ int VariantPropertyManager::valueType(int propertyType) const
     if (propertyType == alignmentTypeId())
         return propertyType;
     if (propertyType == unstyledGroupTypeId())
-        return propertyType;
+        return QMetaType::QVariantMap; // allows storing any child values
     return QtVariantPropertyManager::valueType(propertyType);
 }
 
@@ -166,15 +166,15 @@ int VariantPropertyManager::unstyledGroupTypeId()
     return qMetaTypeId<UnstyledGroup>();
 }
 
-QString VariantPropertyManager::objectRefLabel(const MapObject *object) const
+QString VariantPropertyManager::objectRefLabel(const MapObject &object)
 {
-    QString label = tr("%1: ").arg(object->id());
-    if (!object->name().isEmpty()) {
-        label.append(object->name());
-        if (!object->type().isEmpty())
-            label.append(tr(" (%1)").arg(object->type()));
-    } else if (!object->type().isEmpty())
-        label.append(tr("(%1)").arg(object->type()));
+    QString label = tr("%1: ").arg(object.id());
+    if (!object.name().isEmpty()) {
+        label.append(object.name());
+        if (!object.type().isEmpty())
+            label.append(tr(" (%1)").arg(object.type()));
+    } else if (!object.type().isEmpty())
+        label.append(tr("(%1)").arg(object.type()));
     else
         label.append(tr("Unnamed object"));
 
@@ -194,7 +194,7 @@ QString VariantPropertyManager::valueText(const QtProperty *property) const
                 return tr("Unset");
 
             if (auto object = ref.object())
-                return objectRefLabel(object);
+                return objectRefLabel(*object);
 
             return tr("%1: Object not found").arg(QString::number(ref.id()));
         }
@@ -237,9 +237,19 @@ QString VariantPropertyManager::valueText(const QtProperty *property) const
 QIcon VariantPropertyManager::valueIcon(const QtProperty *property) const
 {
     if (mValues.contains(property)) {
-        QVariant value = mValues[property];
+        const QVariant value = mValues[property];
+        const int typeId = propertyType(property);
+
+        if (typeId == displayObjectRefTypeId()) {
+            const DisplayObjectRef ref = value.value<DisplayObjectRef>();
+            if (auto object = ref.object())
+                return ObjectIconManager::instance().iconForObject(*object);
+        }
+
+        if (typeId == unstyledGroupTypeId())
+            return QIcon();
+
         QString filePath;
-        int typeId = propertyType(property);
 
         // TODO: Needs a special icon for remote files
         if (typeId == filePathTypeId()) {
@@ -252,12 +262,6 @@ QIcon VariantPropertyManager::valueIcon(const QtProperty *property) const
                 filePath = tilesetDocument->tileset()->imageSource().toLocalFile();
         }
 
-        if (typeId == displayObjectRefTypeId()) {
-            const DisplayObjectRef ref = value.value<DisplayObjectRef>();
-            if (auto object = ref.object())
-                return ObjectIconManager::instance().iconForObject(object);
-        }
-
         // TODO: This assumes the file path is an image reference. It should be
         // replaced with a more generic icon.
         if (filePath.isEmpty() || !QFile::exists(filePath))
@@ -350,7 +354,8 @@ void VariantPropertyManager::initializeProperty(QtProperty *property)
     const int type = propertyType(property);
     if (type == filePathTypeId()
             || type == displayObjectRefTypeId()
-            || type == tilesetParametersTypeId()) {
+            || type == tilesetParametersTypeId()
+            || type == unstyledGroupTypeId()) { // for storing hash map
         mValues[property] = QVariant();
         if (type == filePathTypeId())
             mFilePathAttributes[property] = FilePathAttributes();
