@@ -53,8 +53,8 @@ public:
 class UnstyledGroup {};
 
 /**
- * Extension of the QtVariantPropertyManager that adds support for a filePath
- * data type.
+ * Extension of the QtVariantPropertyManager that adds support for various
+ * additional types and attributes.
  */
 class VariantPropertyManager : public QtVariantPropertyManager
 {
@@ -92,7 +92,8 @@ protected:
 private:
     void slotValueChanged(QtProperty *property, const QVariant &value);
     void slotPropertyDestroyed(QtProperty *property);
-    QString objectRefLabel(const MapObject *object) const;
+
+    static QString objectRefLabel(const MapObject &object);
 
     QMap<const QtProperty *, QVariant> mValues;
 
@@ -115,7 +116,7 @@ private:
     QString indexHToString(int idx) const;
     QString indexVToString(int idx) const;
     QMap<const QtProperty *, Qt::Alignment> m_alignValues;
-    typedef QMap<QtProperty *, QtProperty *> PropertyToPropertyMap;
+    using PropertyToPropertyMap = QMap<QtProperty *, QtProperty *>;
     PropertyToPropertyMap m_propertyToAlignH;
     PropertyToPropertyMap m_propertyToAlignV;
     PropertyToPropertyMap m_alignHToProperty;
