@@ -20,16 +20,16 @@
 
 #include "wangsetmodel.h"
 
+#include "changeevents.h"
 #include "containerhelpers.h"
 #include "map.h"
 #include "mapdocument.h"
-#include "wangset.h"
-#include "tileset.h"
 #include "tile.h"
+#include "tileset.h"
 #include "tilesetdocument.h"
 #include "tilesetdocumentsmodel.h"
-#include "tileset.h"
-#include "tilesetwangsetmodel.h"
+#include "wangoverlay.h"
+#include "wangset.h"
 
 #include <QApplication>
 #include <QFont>
@@ -121,6 +121,8 @@ QVariant WangSetModel::data(const QModelIndex &index, int role) const
         case Qt::DecorationRole:
             if (Tile *tile = wangSet->imageTile())
                 return tile->image();
+            else
+                return wangSetIcon(wangSet->type());
             break;
         case WangSetRole:
             return QVariant::fromValue(wangSet);
@@ -189,15 +191,7 @@ void WangSetModel::onTilesetRowsInserted(const QModelIndex &parent, int first, i
 
         mTilesetDocuments.insert(row, tilesetDocument);
 
-        TilesetWangSetModel *tilesetWangSetModel = tilesetDocument->wangSetModel();
-        connect(tilesetWangSetModel, &TilesetWangSetModel::wangSetAboutToBeAdded,
-                this, &WangSetModel::onWangSetAboutToBeAdded);
-        connect(tilesetWangSetModel, &TilesetWangSetModel::wangSetAdded,
-                this, &WangSetModel::onWangSetAdded);
-        connect(tilesetWangSetModel, &TilesetWangSetModel::wangSetAboutToBeRemoved,
-                this, &WangSetModel::onWangSetAboutToBeRemoved);
-        connect(tilesetWangSetModel, &TilesetWangSetModel::wangSetRemoved,
-                this, &WangSetModel::onWangSetRemoved);
+        connect(tilesetDocument, &Document::changed, this, &WangSetModel::onDocumentChanged);
     }
     endInsertRows();
 }
@@ -209,7 +203,7 @@ void WangSetModel::onTilesetRowsAboutToBeRemoved(const QModelIndex &parent, int
     beginRemoveRows(QModelIndex(), first, last);
     for (int index = last; index >= first; --index) {
         TilesetDocument *tilesetDocument = mTilesetDocuments.takeAt(index);
-        tilesetDocument->wangSetModel()->disconnect(this);
+        tilesetDocument->disconnect(this);
     }
     endRemoveRows();
 }
@@ -262,30 +256,46 @@ void WangSetModel::onTilesetDataChanged(const QModelIndex &topLeft, const QModel
                      index(bottomRight.row(), bottomRight.column()));
 }
 
-void WangSetModel::onWangSetAboutToBeAdded(Tileset *tileset)
+void WangSetModel::onDocumentChanged(const ChangeEvent &change)
 {
-    QModelIndex parent = index(tileset);
-    beginInsertRows(parent, tileset->wangSetCount(), tileset->wangSetCount());
+    switch (change.type) {
+    case ChangeEvent::WangSetAboutToBeAdded: {
+        auto wangSetEvent = static_cast<const WangSetEvent&>(change);
+
+        QModelIndex parent = this->index(wangSetEvent.tileset);
+        beginInsertRows(parent, wangSetEvent.index, wangSetEvent.index);
+        break;
+    }
+    case ChangeEvent::WangSetAdded: {
+        auto wangSetEvent = static_cast<const WangSetEvent&>(change);
+
+        endInsertRows();
+
+        const QModelIndex index = WangSetModel::index(wangSetEvent.tileset);
+        emit dataChanged(index, index);
+        break;
+    }
+
+    case ChangeEvent::WangSetAboutToBeRemoved: {
+        auto wangSetEvent = static_cast<const WangSetEvent&>(change);
+        auto wangSet = wangSetEvent.tileset->wangSet(wangSetEvent.index);
+
+        QModelIndex parent = index(wangSetEvent.tileset);
+        beginRemoveRows(parent, index(wangSet).row(), index(wangSet).row());
+        break;
+    }
+    case ChangeEvent::WangSetRemoved: {
+        auto wangSetEvent = static_cast<const WangSetEvent&>(change);
+
+        endRemoveRows();
+
+        const QModelIndex index = WangSetModel::index(wangSetEvent.tileset);
+        emit dataChanged(index, index);
+        break;
+    }
+    default:
+        break;
+    }
 }
 
-void WangSetModel::onWangSetAdded(Tileset *tileset)
-{
-    endInsertRows();
-
-    const QModelIndex index = WangSetModel::index(tileset);
-    emit dataChanged(index, index);
-}
-
-void WangSetModel::onWangSetAboutToBeRemoved(WangSet *wangSet)
-{
-    QModelIndex parent = index(wangSet->tileset());
-    beginRemoveRows(parent, index(wangSet).row(), index(wangSet).row());
-}
-
-void WangSetModel::onWangSetRemoved(WangSet *wangSet)
-{
-    endRemoveRows();
-
-    const QModelIndex index = WangSetModel::index(wangSet->tileset());
-    emit dataChanged(index, index);
-}
+#include "moc_wangsetmodel.cpp"
