@@ -42,7 +42,6 @@ class QPainter;
 
 class TmxRasterizer
 {
-
 public:
     TmxRasterizer();
 
@@ -68,17 +67,17 @@ public:
     int render(const QString &fileName, const QString &imageFileName);
 
 private:
-    qreal mScale;
-    int mTileSize;
-    int mSize;
-    int mAdvanceAnimations;
-    bool mUseAntiAliasing;
-    bool mSmoothImages;
-    bool mIgnoreVisibility;
+    qreal mScale = 1.0;
+    int mTileSize = 0;
+    int mSize = 0;
+    int mAdvanceAnimations = 0;
+    bool mUseAntiAliasing = false;
+    bool mSmoothImages = true;
+    bool mIgnoreVisibility = false;
     QStringList mLayersToHide;
     QStringList mLayersToShow;
 
-    void drawMapLayers(MapRenderer &renderer, QPainter &painter, Map &map, QPoint mapOffset = QPoint(0, 0)) const;
+    void drawMapLayers(const MapRenderer &renderer, QPainter &painter, QPoint mapOffset = QPoint(0, 0)) const;
     int renderMap(const QString &mapFileName, const QString &imageFileName);
     int renderWorld(const QString &worldFileName, const QString &imageFileName);
     int saveImage(const QString &imageFileName, const QImage &image) const;
